# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from .schemas import (
    SchemasResource,
    AsyncSchemasResource,
    SchemasResourceWithRawResponse,
    AsyncSchemasResourceWithRawResponse,
    SchemasResourceWithStreamingResponse,
    AsyncSchemasResourceWithStreamingResponse,
)
from .settings import (
    SettingsResource,
    AsyncSettingsResource,
    SettingsResourceWithRawResponse,
    AsyncSettingsResourceWithRawResponse,
    SettingsResourceWithStreamingResponse,
    AsyncSettingsResourceWithStreamingResponse,
)
from .schema_validation import (
    SchemaValidationResource,
    AsyncSchemaValidationResource,
    SchemaValidationResourceWithRawResponse,
    AsyncSchemaValidationResourceWithRawResponse,
    SchemaValidationResourceWithStreamingResponse,
    AsyncSchemaValidationResourceWithStreamingResponse,
)

__all__ = [
    "SchemasResource",
    "AsyncSchemasResource",
    "SchemasResourceWithRawResponse",
    "AsyncSchemasResourceWithRawResponse",
    "SchemasResourceWithStreamingResponse",
    "AsyncSchemasResourceWithStreamingResponse",
    "SettingsResource",
    "AsyncSettingsResource",
    "SettingsResourceWithRawResponse",
    "AsyncSettingsResourceWithRawResponse",
    "SettingsResourceWithStreamingResponse",
    "AsyncSettingsResourceWithStreamingResponse",
    "SchemaValidationResource",
    "AsyncSchemaValidationResource",
    "SchemaValidationResourceWithRawResponse",
    "AsyncSchemaValidationResourceWithRawResponse",
    "SchemaValidationResourceWithStreamingResponse",
    "AsyncSchemaValidationResourceWithStreamingResponse",
]



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from __future__ import annotations

import typing_extensions
from typing import Type, Union, cast
from datetime import datetime
from typing_extensions import Literal

import httpx

from ....._types import Body, Omit, Query, Headers, NotGiven, SequenceNotStr, omit, not_given
from ....._utils import maybe_transform, async_maybe_transform
from ....._compat import cached_property
from ....._resource import SyncAPIResource, AsyncAPIResource
from ....._response import (
    to_raw_response_wrapper,
    to_streamed_response_wrapper,
    async_to_raw_response_wrapper,
    async_to_streamed_response_wrapper,
)
from ....._wrappers import ResultWrapper
from ....._base_client import make_request_options
from .....types.radar.ai.inference import summary_task_params, summary_model_params
from .....types.radar.ai.inference.summary_task_response import SummaryTaskResponse
from .....types.radar.ai.inference.summary_model_response import SummaryModelResponse

__all__ = ["SummaryResource", "AsyncSummaryResource"]


class SummaryResource(SyncAPIResource):
    @cached_property
    def with_raw_response(self) -> SummaryResourceWithRawResponse:
        """
        This property can be used as a prefix for any HTTP method call to return
        the raw response object instead of the parsed content.

        For more information, see https://www.github.com/cloudflare/cloudflare-python#accessing-raw-response-data-eg-headers
        """
        return SummaryResourceWithRawResponse(self)

    @cached_property
    def with_streaming_response(self) -> SummaryResourceWithStreamingResponse:
        """
        An alternative to `.with_raw_response` that doesn't eagerly read the response body.

        For more information, see https://www.github.com/cloudflare/cloudflare-python#with_streaming_response
        """
        return SummaryResourceWithStreamingResponse(self)

    @typing_extensions.deprecated(
        "Use [Radar AI Inference Summary By Dimension](https://developers.cloudflare.com/api/resources/radar/subresources/ai/subresources/inference/methods/summary_v2/) instead."
    )
    def model(
        self,
        *,
        date_end: SequenceNotStr[Union[str, datetime]] | Omit = omit,
        date_range: SequenceNotStr[str] | Omit = omit,
        date_start: SequenceNotStr[Union[str, datetime]] | Omit = omit,
        format: Literal["JSON", "CSV"] | Omit = omit,
        limit_per_group: int | Omit = omit,
        name: SequenceNotStr[str] | Omit = omit,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> SummaryModelResponse:
        """
        Retrieves the distribution of unique accounts by model.

        Args:
          date_end: End of the date range (inclusive).

          date_range: Filters results by date range. For example, use `7d` and `7dcontrol` to compare
              this week with the previous week. Use this parameter or set specific start and
              end dates (`dateStart` and `dateEnd` parameters).

          date_start: Start of the date range.

          format: Format in which results will be returned.

          limit_per_group: Limits the number of objects per group to the top items within the specified
              time range. When item count exceeds the limit, extra items appear grouped under
              an "other" category.

          name: Array of names used to label the series in the response.

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        return self._get(
            "/radar/ai/inference/summary/model",
            options=make_request_options(
                extra_headers=extra_headers,
                extra_query=extra_query,
                extra_body=extra_body,
                timeout=timeout,
                query=maybe_transform(
                    {
                        "date_end": date_end,
                        "date_range": date_range,
                        "date_start": date_start,
                        "format": format,
                        "limit_per_group": limit_per_group,
                        "name": name,
                    },
                    summary_model_params.SummaryModelParams,
                ),
                post_parser=ResultWrapper[SummaryModelResponse]._unwrapper,
            ),
            cast_to=cast(Type[SummaryModelResponse], ResultWrapper[SummaryModelResponse]),
        )

    @typing_extensions.deprecated(
        "Use [Radar AI Inference Summary By Dimension](https://developers.cloudflare.com/api/resources/radar/subresources/ai/subresources/inference/methods/summary_v2/) instead."
    )
    def task(
        self,
        *,
        date_end: SequenceNotStr[Union[str, datetime]] | Omit = omit,
        date_range: SequenceNotStr[str] | Omit = omit,
        date_start: SequenceNotStr[Union[str, datetime]] | Omit = omit,
        format: Literal["JSON", "CSV"] | Omit = omit,
        limit_per_group: int | Omit = omit,
        name: SequenceNotStr[str] | Omit = omit,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> SummaryTaskResponse:
        """
        Retrieves the distribution of unique accounts by task.

        Args:
          date_end: End of the date range (inclusive).

          date_range: Filters results by date range. For example, use `7d` and `7dcontrol` to compare
              this week with the previous week. Use this parameter or set specific start and
              end dates (`dateStart` and `dateEnd` parameters).

          date_start: Start of the date range.

          format: Format in which results will be returned.

          limit_per_group: Limits the number of objects per group to the top items within the specified
              time range. When item count exceeds the limit, extra items appear grouped under
              an "other" category.

          name: Array of names used to label the series in the response.

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        return self._get(
            "/radar/ai/inference/summary/task",
            options=make_request_options(
                extra_headers=extra_headers,
                extra_query=extra_query,
                extra_body=extra_body,
                timeout=timeout,
                query=maybe_transform(
                    {
                        "date_end": date_end,
                        "date_range": date_range,
                        "date_start": date_start,
                        "format": format,
                        "limit_per_group": limit_per_group,
                        "name": name,
                    },
                    summary_task_params.SummaryTaskParams,
                ),
                post_parser=ResultWrapper[SummaryTaskResponse]._unwrapper,
            ),
            cast_to=cast(Type[SummaryTaskResponse], ResultWrapper[SummaryTaskResponse]),
        )


class AsyncSummaryResource(AsyncAPIResource):
    @cached_property
    def with_raw_response(self) -> AsyncSummaryResourceWithRawResponse:
        """
        This property can be used as a prefix for any HTTP method call to return
        the raw response object instead of the parsed content.

        For more information, see https://www.github.com/cloudflare/cloudflare-python#accessing-raw-response-data-eg-headers
        """
        return AsyncSummaryResourceWithRawResponse(self)

    @cached_property
    def with_streaming_response(self) -> AsyncSummaryResourceWithStreamingResponse:
        """
        An alternative to `.with_raw_response` that doesn't eagerly read the response body.

        For more information, see https://www.github.com/cloudflare/cloudflare-python#with_streaming_response
        """
        return AsyncSummaryResourceWithStreamingResponse(self)

    @typing_extensions.deprecated(
        "Use [Radar AI Inference Summary By Dimension](https://developers.cloudflare.com/api/resources/radar/subresources/ai/subresources/inference/methods/summary_v2/) instead."
    )
    async def model(
        self,
        *,
        date_end: SequenceNotStr[Union[str, datetime]] | Omit = omit,
        date_range: SequenceNotStr[str] | Omit = omit,
        date_start: SequenceNotStr[Union[str, datetime]] | Omit = omit,
        format: Literal["JSON", "CSV"] | Omit = omit,
        limit_per_group: int | Omit = omit,
        name: SequenceNotStr[str] | Omit = omit,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> SummaryModelResponse:
        """
        Retrieves the distribution of unique accounts by model.

        Args:
          date_end: End of the date range (inclusive).

          date_range: Filters results by date range. For example, use `7d` and `7dcontrol` to compare
              this week with the previous week. Use this parameter or set specific start and
              end dates (`dateStart` and `dateEnd` parameters).

          date_start: Start of the date range.

          format: Format in which results will be returned.

          limit_per_group: Limits the number of objects per group to the top items within the specified
              time range. When item count exceeds the limit, extra items appear grouped under
              an "other" category.

          name: Array of names used to label the series in the response.

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        return await self._get(
            "/radar/ai/inference/summary/model",
            options=make_request_options(
                extra_headers=extra_headers,
                extra_query=extra_query,
                extra_body=extra_body,
                timeout=timeout,
                query=await async_maybe_transform(
                    {
                        "date_end": date_end,
                        "date_range": date_range,
                        "date_start": date_start,
                        "format": format,
                        "limit_per_group": limit_per_group,
                        "name": name,
                    },
                    summary_model_params.SummaryModelParams,
                ),
                post_parser=ResultWrapper[SummaryModelResponse]._unwrapper,
            ),
            cast_to=cast(Type[SummaryModelResponse], ResultWrapper[SummaryModelResponse]),
        )

    @typing_extensions.deprecated(
        "Use [Radar AI Inference Summary By Dimension](https://developers.cloudflare.com/api/resources/radar/subresources/ai/subresources/inference/methods/summary_v2/) instead."
    )
    async def task(
        self,
        *,
        date_end: SequenceNotStr[Union[str, datetime]] | Omit = omit,
        date_range: SequenceNotStr[str] | Omit = omit,
        date_start: SequenceNotStr[Union[str, datetime]] | Omit = omit,
        format: Literal["JSON", "CSV"] | Omit = omit,
        limit_per_group: int | Omit = omit,
        name: SequenceNotStr[str] | Omit = omit,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> SummaryTaskResponse:
        """
        Retrieves the distribution of unique accounts by task.

        Args:
          date_end: End of the date range (inclusive).

          date_range: Filters results by date range. For example, use `7d` and `7dcontrol` to compare
              this week with the previous week. Use this parameter or set specific start and
              end dates (`dateStart` and `dateEnd` parameters).

          date_start: Start of the date range.

          format: Format in which results will be returned.

          limit_per_group: Limits the number of objects per group to the top items within the specified
              time range. When item count exceeds the limit, extra items appear grouped under
              an "other" category.

          name: Array of names used to label the series in the response.

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        return await self._get(
            "/radar/ai/inference/summary/task",
            options=make_request_options(
                extra_headers=extra_headers,
                extra_query=extra_query,
                extra_body=extra_body,
                timeout=timeout,
                query=await async_maybe_transform(
                    {
                        "date_end": date_end,
                        "date_range": date_range,
                        "date_start": date_start,
                        "format": format,
                        "limit_per_group": limit_per_group,
                        "name": name,
                    },
                    summary_task_params.SummaryTaskParams,
                ),
                post_parser=ResultWrapper[SummaryTaskResponse]._unwrapper,
            ),
            cast_to=cast(Type[SummaryTaskResponse], ResultWrapper[SummaryTaskResponse]),
        )


class SummaryResourceWithRawResponse:
    def __init__(self, summary: SummaryResource) -> None:
        self._summary = summary

        self.model = (  # pyright: ignore[reportDeprecated]
            to_raw_response_wrapper(
                summary.model,  # pyright: ignore[reportDeprecated],
            )
        )
        self.task = (  # pyright: ignore[reportDeprecated]
            to_raw_response_wrapper(
                summary.task,  # pyright: ignore[reportDeprecated],
            )
        )


class AsyncSummaryResourceWithRawResponse:
    def __init__(self, summary: AsyncSummaryResource) -> None:
        self._summary = summary

        self.model = (  # pyright: ignore[reportDeprecated]
            async_to_raw_response_wrapper(
                summary.model,  # pyright: ignore[reportDeprecated],
            )
        )
        self.task = (  # pyright: ignore[reportDeprecated]
            async_to_raw_response_wrapper(
                summary.task,  # pyright: ignore[reportDeprecated],
            )
        )


class SummaryResourceWithStreamingResponse:
    def __init__(self, summary: SummaryResource) -> None:
        self._summary = summary

        self.model = (  # pyright: ignore[reportDeprecated]
            to_streamed_response_wrapper(
                summary.model,  # pyright: ignore[reportDeprecated],
            )
        )
        self.task = (  # pyright: ignore[reportDeprecated]
            to_streamed_response_wrapper(
                summary.task,  # pyright: ignore[reportDeprecated],
            )
        )


class AsyncSummaryResourceWithStreamingResponse:
    def __init__(self, summary: AsyncSummaryResource) -> None:
        self._summary = summary

        self.model = (  # pyright: ignore[reportDeprecated]
            async_to_streamed_response_wrapper(
                summary.model,  # pyright: ignore[reportDeprecated],
            )
        )
        self.task = (  # pyright: ignore[reportDeprecated]
            async_to_streamed_response_wrapper(
                summary.task,  # pyright: ignore[reportDeprecated],
            )
        )



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from __future__ import annotations

from typing_extensions import Required, TypedDict

from ...._types import SequenceNotStr

__all__ = ["FallbackDomainParam"]


class FallbackDomainParam(TypedDict, total=False):
    suffix: Required[str]
    """The domain suffix to match when resolving locally."""

    description: str
    """A description of the fallback domain, displayed in the client UI."""

    dns_server: SequenceNotStr[str]
    """A list of IP addresses to handle domain resolution."""



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from typing import List

from ....._models import BaseModel

__all__ = ["EventListResponse", "ASNInfo", "Event", "EventTag"]


class ASNInfo(BaseModel):
    asn: int

    country_code: str

    org_name: str


class EventTag(BaseModel):
    name: str

    score: int


class Event(BaseModel):
    id: int

    confidence_score: int

    duration: int

    event_type: int

    hijack_msgs_count: int

    hijacker_asn: int

    hijacker_country: str

    is_stale: bool

    max_hijack_ts: str

    max_msg_ts: str

    min_hijack_ts: str

    on_going_count: int

    peer_asns: List[int]

    peer_ip_count: int

    prefixes: List[str]

    tags: List[EventTag]

    victim_asns: List[int]

    victim_countries: List[str]


class EventListResponse(BaseModel):
    asn_info: List[ASNInfo]

    events: List[Event]

    total_monitors: int



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from __future__ import annotations

from typing import List, Type, Optional, cast
from typing_extensions import Literal

import httpx

from ...._types import Body, Omit, Query, Headers, NotGiven, omit, not_given
from ...._utils import path_template, maybe_transform, async_maybe_transform
from ...._compat import cached_property
from ...._resource import SyncAPIResource, AsyncAPIResource
from ...._response import (
    to_raw_response_wrapper,
    to_streamed_response_wrapper,
    async_to_raw_response_wrapper,
    async_to_streamed_response_wrapper,
)
from ...._wrappers import ResultWrapper
from ....pagination import SyncSinglePage, AsyncSinglePage
from ...._base_client import AsyncPaginator, make_request_options
from ....types.zero_trust.gateway import rule_create_params, rule_update_params
from ....types.zero_trust.gateway.gateway_rule import GatewayRule
from ....types.zero_trust.gateway.gateway_filter import GatewayFilter
from ....types.zero_trust.gateway.schedule_param import ScheduleParam
from ....types.zero_trust.gateway.rule_setting_param import RuleSettingParam

__all__ = ["RulesResource", "AsyncRulesResource"]


class RulesResource(SyncAPIResource):
    @cached_property
    def with_raw_response(self) -> RulesResourceWithRawResponse:
        """
        This property can be used as a prefix for any HTTP method call to return
        the raw response object instead of the parsed content.

        For more information, see https://www.github.com/cloudflare/cloudflare-python#accessing-raw-response-data-eg-headers
        """
        return RulesResourceWithRawResponse(self)

    @cached_property
    def with_streaming_response(self) -> RulesResourceWithStreamingResponse:
        """
        An alternative to `.with_raw_response` that doesn't eagerly read the response body.

        For more information, see https://www.github.com/cloudflare/cloudflare-python#with_streaming_response
        """
        return RulesResourceWithStreamingResponse(self)

    def create(
        self,
        *,
        account_id: str,
        action: Literal[
            "on",
            "off",
            "allow",
            "block",
            "scan",
            "noscan",
            "safesearch",
            "ytrestricted",
            "isolate",
            "noisolate",
            "override",
            "l4_override",
            "egress",
            "resolve",
            "quarantine",
            "redirect",
        ],
        name: str,
        description: str | Omit = omit,
        device_posture: str | Omit = omit,
        enabled: bool | Omit = omit,
        expiration: Optional[rule_create_params.Expiration] | Omit = omit,
        filters: List[GatewayFilter] | Omit = omit,
        identity: str | Omit = omit,
        precedence: int | Omit = omit,
        rule_settings: RuleSettingParam | Omit = omit,
        schedule: Optional[ScheduleParam] | Omit = omit,
        traffic: str | Omit = omit,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> Optional[GatewayRule]:
        """
        Create a new Zero Trust Gateway rule.

        Args:
          action: Specify the action to perform when the associated traffic, identity, and device
              posture expressions either absent or evaluate to `true`.

          name: Specify the rule name.

          description: Specify the rule description.

          device_posture: Specify the wirefilter expression used for device posture check. The API
              automatically formats and sanitizes expressions before storing them. To prevent
              Terraform state drift, use the formatted expression returned in the API
              response.

          enabled: Specify whether the rule is enabled.

          expiration: Defines the expiration time stamp and default duration of a DNS policy. Takes
              precedence over the policy's `schedule` configuration, if any. This does not
              apply to HTTP or network policies. Settable only for `dns` rules.

          filters: Specify the protocol or layer to evaluate the traffic, identity, and device
              posture expressions. Can only contain a single value.

          identity: Specify the wirefilter expression used for identity matching. The API
              automatically formats and sanitizes expressions before storing them. To prevent
              Terraform state drift, use the formatted expression returned in the API
              response.

          precedence: Set the order of your rules. Lower values indicate higher precedence. At each
              processing phase, evaluate applicable rules in ascending order of this value.
              Refer to
              [Order of enforcement](http://developers.cloudflare.com/learning-paths/secure-internet-traffic/understand-policies/order-of-enforcement/#manage-precedence-with-terraform)
              to manage precedence via Terraform.

          rule_settings: Defines settings for this rule. Settings apply only to specific rule types and
              must use compatible selectors. If Terraform detects drift, confirm the setting
              supports your rule type and check whether the API modifies the value. Use
              API-returned values in your configuration to prevent drift.

          schedule: Defines the schedule for activating DNS policies. Settable only for `dns` and
              `dns_resolver` rules.

          traffic: Specify the wirefilter expression used for traffic matching. The API
              automatically formats and sanitizes expressions before storing them. To prevent
              Terraform state drift, use the formatted expression returned in the API
              response.

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not account_id:
            raise ValueError(f"Expected a non-empty value for `account_id` but received {account_id!r}")
        return self._post(
            path_template("/accounts/{account_id}/gateway/rules", account_id=account_id),
            body=maybe_transform(
                {
                    "action": action,
                    "name": name,
                    "description": description,
                    "device_posture": device_posture,
                    "enabled": enabled,
                    "expiration": expiration,
                    "filters": filters,
                    "identity": identity,
                    "precedence": precedence,
                    "rule_settings": rule_settings,
                    "schedule": schedule,
                    "traffic": traffic,
                },
                rule_create_params.RuleCreateParams,
            ),
            options=make_request_options(
                extra_headers=extra_headers,
                extra_query=extra_query,
                extra_body=extra_body,
                timeout=timeout,
                post_parser=ResultWrapper[Optional[GatewayRule]]._unwrapper,
            ),
            cast_to=cast(Type[Optional[GatewayRule]], ResultWrapper[GatewayRule]),
        )

    def update(
        self,
        rule_id: str,
        *,
        account_id: str,
        action: Literal[
            "on",
            "off",
            "allow",
            "block",
            "scan",
            "noscan",
            "safesearch",
            "ytrestricted",
            "isolate",
            "noisolate",
            "override",
            "l4_override",
            "egress",
            "resolve",
            "quarantine",
            "redirect",
        ],
        name: str,
        description: str | Omit = omit,
        device_posture: str | Omit = omit,
        enabled: bool | Omit = omit,
        expiration: Optional[rule_update_params.Expiration] | Omit = omit,
        filters: List[GatewayFilter] | Omit = omit,
        identity: str | Omit = omit,
        precedence: int | Omit = omit,
        rule_settings: RuleSettingParam | Omit = omit,
        schedule: Optional[ScheduleParam] | Omit = omit,
        traffic: str | Omit = omit,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> Optional[GatewayRule]:
        """
        Update a configured Zero Trust Gateway rule.

        Args:
          rule_id: Identify the API resource with a UUID.

          action: Specify the action to perform when the associated traffic, identity, and device
              posture expressions either absent or evaluate to `true`.

          name: Specify the rule name.

          description: Specify the rule description.

          device_posture: Specify the wirefilter expression used for device posture check. The API
              automatically formats and sanitizes expressions before storing them. To prevent
              Terraform state drift, use the formatted expression returned in the API
              response.

          enabled: Specify whether the rule is enabled.

          expiration: Defines the expiration time stamp and default duration of a DNS policy. Takes
              precedence over the policy's `schedule` configuration, if any. This does not
              apply to HTTP or network policies. Settable only for `dns` rules.

          filters: Specify the protocol or layer to evaluate the traffic, identity, and device
              posture expressions. Can only contain a single value.

          identity: Specify the wirefilter expression used for identity matching. The API
              automatically formats and sanitizes expressions before storing them. To prevent
              Terraform state drift, use the formatted expression returned in the API
              response.

          precedence: Set the order of your rules. Lower values indicate higher precedence. At each
              processing phase, evaluate applicable rules in ascending order of this value.
              Refer to
              [Order of enforcement](http://developers.cloudflare.com/learning-paths/secure-internet-traffic/understand-policies/order-of-enforcement/#manage-precedence-with-terraform)
              to manage precedence via Terraform.

          rule_settings: Defines settings for this rule. Settings apply only to specific rule types and
              must use compatible selectors. If Terraform detects drift, confirm the setting
              supports your rule type and check whether the API modifies the value. Use
              API-returned values in your configuration to prevent drift.

          schedule: Defines the schedule for activating DNS policies. Settable only for `dns` and
              `dns_resolver` rules.

          traffic: Specify the wirefilter expression used for traffic matching. The API
              automatically formats and sanitizes expressions before storing them. To prevent
              Terraform state drift, use the formatted expression returned in the API
              response.

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not account_id:
            raise ValueError(f"Expected a non-empty value for `account_id` but received {account_id!r}")
        if not rule_id:
            raise ValueError(f"Expected a non-empty value for `rule_id` but received {rule_id!r}")
        return self._put(
            path_template("/accounts/{account_id}/gateway/rules/{rule_id}", account_id=account_id, rule_id=rule_id),
            body=maybe_transform(
                {
                    "action": action,
                    "name": name,
                    "description": description,
                    "device_posture": device_posture,
                    "enabled": enabled,
                    "expiration": expiration,
                    "filters": filters,
                    "identity": identity,
                    "precedence": precedence,
                    "rule_settings": rule_settings,
                    "schedule": schedule,
                    "traffic": traffic,
                },
                rule_update_params.RuleUpdateParams,
            ),
            options=make_request_options(
                extra_headers=extra_headers,
                extra_query=extra_query,
                extra_body=extra_body,
                timeout=timeout,
                post_parser=ResultWrapper[Optional[GatewayRule]]._unwrapper,
            ),
            cast_to=cast(Type[Optional[GatewayRule]], ResultWrapper[GatewayRule]),
        )

    def list(
        self,
        *,
        account_id: str,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> SyncSinglePage[GatewayRule]:
        """
        List Zero Trust Gateway rules for an account.

        Args:
          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not account_id:
            raise ValueError(f"Expected a non-empty value for `account_id` but received {account_id!r}")
        return self._get_api_list(
            path_template("/accounts/{account_id}/gateway/rules", account_id=account_id),
            page=SyncSinglePage[GatewayRule],
            options=make_request_options(
                extra_headers=extra_headers, extra_query=extra_query, extra_body=extra_body, timeout=timeout
            ),
            model=GatewayRule,
        )

    def delete(
        self,
        rule_id: str,
        *,
        account_id: str,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> object:
        """
        Delete a Zero Trust Gateway rule.

        Args:
          rule_id: Identify the API resource with a UUID.

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not account_id:
            raise ValueError(f"Expected a non-empty value for `account_id` but received {account_id!r}")
        if not rule_id:
            raise ValueError(f"Expected a non-empty value for `rule_id` but received {rule_id!r}")
        return self._delete(
            path_template("/accounts/{account_id}/gateway/rules/{rule_id}", account_id=account_id, rule_id=rule_id),
            options=make_request_options(
                extra_headers=extra_headers,
                extra_query=extra_query,
                extra_body=extra_body,
                timeout=timeout,
                post_parser=ResultWrapper[Optional[object]]._unwrapper,
            ),
            cast_to=cast(Type[object], ResultWrapper[object]),
        )

    def get(
        self,
        rule_id: str,
        *,
        account_id: str,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> Optional[GatewayRule]:
        """
        Get a single Zero Trust Gateway rule.

        Args:
          rule_id: Identify the API resource with a UUID.

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not account_id:
            raise ValueError(f"Expected a non-empty value for `account_id` but received {account_id!r}")
        if not rule_id:
            raise ValueError(f"Expected a non-empty value for `rule_id` but received {rule_id!r}")
        return self._get(
            path_template("/accounts/{account_id}/gateway/rules/{rule_id}", account_id=account_id, rule_id=rule_id),
            options=make_request_options(
                extra_headers=extra_headers,
                extra_query=extra_query,
                extra_body=extra_body,
                timeout=timeout,
                post_parser=ResultWrapper[Optional[GatewayRule]]._unwrapper,
            ),
            cast_to=cast(Type[Optional[GatewayRule]], ResultWrapper[GatewayRule]),
        )

    def list_tenant(
        self,
        *,
        account_id: str,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> SyncSinglePage[GatewayRule]:
        """
        List Zero Trust Gateway rules for the parent account of an account in the MSP
        configuration.

        Args:
          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not account_id:
            raise ValueError(f"Expected a non-empty value for `account_id` but received {account_id!r}")
        return self._get_api_list(
            path_template("/accounts/{account_id}/gateway/rules/tenant", account_id=account_id),
            page=SyncSinglePage[GatewayRule],
            options=make_request_options(
                extra_headers=extra_headers, extra_query=extra_query, extra_body=extra_body, timeout=timeout
            ),
            model=GatewayRule,
        )

    def reset_expiration(
        self,
        rule_id: str,
        *,
        account_id: str,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> Optional[GatewayRule]:
        """
        Resets the expiration of a Zero Trust Gateway Rule if its duration elapsed and
        it has a default duration. The Zero Trust Gateway Rule must have values for both
        `expiration.expires_at` and `expiration.duration`.

        Args:
          rule_id: Identify the API resource with a UUID.

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not account_id:
            raise ValueError(f"Expected a non-empty value for `account_id` but received {account_id!r}")
        if not rule_id:
            raise ValueError(f"Expected a non-empty value for `rule_id` but received {rule_id!r}")
        return self._post(
            path_template(
                "/accounts/{account_id}/gateway/rules/{rule_id}/reset_expiration",
                account_id=account_id,
                rule_id=rule_id,
            ),
            options=make_request_options(
                extra_headers=extra_headers,
                extra_query=extra_query,
                extra_body=extra_body,
                timeout=timeout,
                post_parser=ResultWrapper[Optional[GatewayRule]]._unwrapper,
            ),
            cast_to=cast(Type[Optional[GatewayRule]], ResultWrapper[GatewayRule]),
        )


class AsyncRulesResource(AsyncAPIResource):
    @cached_property
    def with_raw_response(self) -> AsyncRulesResourceWithRawResponse:
        """
        This property can be used as a prefix for any HTTP method call to return
        the raw response object instead of the parsed content.

        For more information, see https://www.github.com/cloudflare/cloudflare-python#accessing-raw-response-data-eg-headers
        """
        return AsyncRulesResourceWithRawResponse(self)

    @cached_property
    def with_streaming_response(self) -> AsyncRulesResourceWithStreamingResponse:
        """
        An alternative to `.with_raw_response` that doesn't eagerly read the response body.

        For more information, see https://www.github.com/cloudflare/cloudflare-python#with_streaming_response
        """
        return AsyncRulesResourceWithStreamingResponse(self)

    async def create(
        self,
        *,
        account_id: str,
        action: Literal[
            "on",
            "off",
            "allow",
            "block",
            "scan",
            "noscan",
            "safesearch",
            "ytrestricted",
            "isolate",
            "noisolate",
            "override",
            "l4_override",
            "egress",
            "resolve",
            "quarantine",
            "redirect",
        ],
        name: str,
        description: str | Omit = omit,
        device_posture: str | Omit = omit,
        enabled: bool | Omit = omit,
        expiration: Optional[rule_create_params.Expiration] | Omit = omit,
        filters: List[GatewayFilter] | Omit = omit,
        identity: str | Omit = omit,
        precedence: int | Omit = omit,
        rule_settings: RuleSettingParam | Omit = omit,
        schedule: Optional[ScheduleParam] | Omit = omit,
        traffic: str | Omit = omit,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> Optional[GatewayRule]:
        """
        Create a new Zero Trust Gateway rule.

        Args:
          action: Specify the action to perform when the associated traffic, identity, and device
              posture expressions either absent or evaluate to `true`.

          name: Specify the rule name.

          description: Specify the rule description.

          device_posture: Specify the wirefilter expression used for device posture check. The API
              automatically formats and sanitizes expressions before storing them. To prevent
              Terraform state drift, use the formatted expression returned in the API
              response.

          enabled: Specify whether the rule is enabled.

          expiration: Defines the expiration time stamp and default duration of a DNS policy. Takes
              precedence over the policy's `schedule` configuration, if any. This does not
              apply to HTTP or network policies. Settable only for `dns` rules.

          filters: Specify the protocol or layer to evaluate the traffic, identity, and device
              posture expressions. Can only contain a single value.

          identity: Specify the wirefilter expression used for identity matching. The API
              automatically formats and sanitizes expressions before storing them. To prevent
              Terraform state drift, use the formatted expression returned in the API
              response.

          precedence: Set the order of your rules. Lower values indicate higher precedence. At each
              processing phase, evaluate applicable rules in ascending order of this value.
              Refer to
              [Order of enforcement](http://developers.cloudflare.com/learning-paths/secure-internet-traffic/understand-policies/order-of-enforcement/#manage-precedence-with-terraform)
              to manage precedence via Terraform.

          rule_settings: Defines settings for this rule. Settings apply only to specific rule types and
              must use compatible selectors. If Terraform detects drift, confirm the setting
              supports your rule type and check whether the API modifies the value. Use
              API-returned values in your configuration to prevent drift.

          schedule: Defines the schedule for activating DNS policies. Settable only for `dns` and
              `dns_resolver` rules.

          traffic: Specify the wirefilter expression used for traffic matching. The API
              automatically formats and sanitizes expressions before storing them. To prevent
              Terraform state drift, use the formatted expression returned in the API
              response.

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not account_id:
            raise ValueError(f"Expected a non-empty value for `account_id` but received {account_id!r}")
        return await self._post(
            path_template("/accounts/{account_id}/gateway/rules", account_id=account_id),
            body=await async_maybe_transform(
                {
                    "action": action,
                    "name": name,
                    "description": description,
                    "device_posture": device_posture,
                    "enabled": enabled,
                    "expiration": expiration,
                    "filters": filters,
                    "identity": identity,
                    "precedence": precedence,
                    "rule_settings": rule_settings,
                    "schedule": schedule,
                    "traffic": traffic,
                },
                rule_create_params.RuleCreateParams,
            ),
            options=make_request_options(
                extra_headers=extra_headers,
                extra_query=extra_query,
                extra_body=extra_body,
                timeout=timeout,
                post_parser=ResultWrapper[Optional[GatewayRule]]._unwrapper,
            ),
            cast_to=cast(Type[Optional[GatewayRule]], ResultWrapper[GatewayRule]),
        )

    async def update(
        self,
        rule_id: str,
        *,
        account_id: str,
        action: Literal[
            "on",
            "off",
            "allow",
            "block",
            "scan",
            "noscan",
            "safesearch",
            "ytrestricted",
            "isolate",
            "noisolate",
            "override",
            "l4_override",
            "egress",
            "resolve",
            "quarantine",
            "redirect",
        ],
        name: str,
        description: str | Omit = omit,
        device_posture: str | Omit = omit,
        enabled: bool | Omit = omit,
        expiration: Optional[rule_update_params.Expiration] | Omit = omit,
        filters: List[GatewayFilter] | Omit = omit,
        identity: str | Omit = omit,
        precedence: int | Omit = omit,
        rule_settings: RuleSettingParam | Omit = omit,
        schedule: Optional[ScheduleParam] | Omit = omit,
        traffic: str | Omit = omit,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> Optional[GatewayRule]:
        """
        Update a configured Zero Trust Gateway rule.

        Args:
          rule_id: Identify the API resource with a UUID.

          action: Specify the action to perform when the associated traffic, identity, and device
              posture expressions either absent or evaluate to `true`.

          name: Specify the rule name.

          description: Specify the rule description.

          device_posture: Specify the wirefilter expression used for device posture check. The API
              automatically formats and sanitizes expressions before storing them. To prevent
              Terraform state drift, use the formatted expression returned in the API
              response.

          enabled: Specify whether the rule is enabled.

          expiration: Defines the expiration time stamp and default duration of a DNS policy. Takes
              precedence over the policy's `schedule` configuration, if any. This does not
              apply to HTTP or network policies. Settable only for `dns` rules.

          filters: Specify the protocol or layer to evaluate the traffic, identity, and device
              posture expressions. Can only contain a single value.

          identity: Specify the wirefilter expression used for identity matching. The API
              automatically formats and sanitizes expressions before storing them. To prevent
              Terraform state drift, use the formatted expression returned in the API
              response.

          precedence: Set the order of your rules. Lower values indicate higher precedence. At each
              processing phase, evaluate applicable rules in ascending order of this value.
              Refer to
              [Order of enforcement](http://developers.cloudflare.com/learning-paths/secure-internet-traffic/understand-policies/order-of-enforcement/#manage-precedence-with-terraform)
              to manage precedence via Terraform.

          rule_settings: Defines settings for this rule. Settings apply only to specific rule types and
              must use compatible selectors. If Terraform detects drift, confirm the setting
              supports your rule type and check whether the API modifies the value. Use
              API-returned values in your configuration to prevent drift.

          schedule: Defines the schedule for activating DNS policies. Settable only for `dns` and
              `dns_resolver` rules.

          traffic: Specify the wirefilter expression used for traffic matching. The API
              automatically formats and sanitizes expressions before storing them. To prevent
              Terraform state drift, use the formatted expression returned in the API
              response.

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not account_id:
            raise ValueError(f"Expected a non-empty value for `account_id` but received {account_id!r}")
        if not rule_id:
            raise ValueError(f"Expected a non-empty value for `rule_id` but received {rule_id!r}")
        return await self._put(
            path_template("/accounts/{account_id}/gateway/rules/{rule_id}", account_id=account_id, rule_id=rule_id),
            body=await async_maybe_transform(
                {
                    "action": action,
                    "name": name,
                    "description": description,
                    "device_posture": device_posture,
                    "enabled": enabled,
                    "expiration": expiration,
                    "filters": filters,
                    "identity": identity,
                    "precedence": precedence,
                    "rule_settings": rule_settings,
                    "schedule": schedule,
                    "traffic": traffic,
                },
                rule_update_params.RuleUpdateParams,
            ),
            options=make_request_options(
                extra_headers=extra_headers,
                extra_query=extra_query,
                extra_body=extra_body,
                timeout=timeout,
                post_parser=ResultWrapper[Optional[GatewayRule]]._unwrapper,
            ),
            cast_to=cast(Type[Optional[GatewayRule]], ResultWrapper[GatewayRule]),
        )

    def list(
        self,
        *,
        account_id: str,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> AsyncPaginator[GatewayRule, AsyncSinglePage[GatewayRule]]:
        """
        List Zero Trust Gateway rules for an account.

        Args:
          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not account_id:
            raise ValueError(f"Expected a non-empty value for `account_id` but received {account_id!r}")
        return self._get_api_list(
            path_template("/accounts/{account_id}/gateway/rules", account_id=account_id),
            page=AsyncSinglePage[GatewayRule],
            options=make_request_options(
                extra_headers=extra_headers, extra_query=extra_query, extra_body=extra_body, timeout=timeout
            ),
            model=GatewayRule,
        )

    async def delete(
        self,
        rule_id: str,
        *,
        account_id: str,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> object:
        """
        Delete a Zero Trust Gateway rule.

        Args:
          rule_id: Identify the API resource with a UUID.

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not account_id:
            raise ValueError(f"Expected a non-empty value for `account_id` but received {account_id!r}")
        if not rule_id:
            raise ValueError(f"Expected a non-empty value for `rule_id` but received {rule_id!r}")
        return await self._delete(
            path_template("/accounts/{account_id}/gateway/rules/{rule_id}", account_id=account_id, rule_id=rule_id),
            options=make_request_options(
                extra_headers=extra_headers,
                extra_query=extra_query,
                extra_body=extra_body,
                timeout=timeout,
                post_parser=ResultWrapper[Optional[object]]._unwrapper,
            ),
            cast_to=cast(Type[object], ResultWrapper[object]),
        )

    async def get(
        self,
        rule_id: str,
        *,
        account_id: str,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> Optional[GatewayRule]:
        """
        Get a single Zero Trust Gateway rule.

        Args:
          rule_id: Identify the API resource with a UUID.

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not account_id:
            raise ValueError(f"Expected a non-empty value for `account_id` but received {account_id!r}")
        if not rule_id:
            raise ValueError(f"Expected a non-empty value for `rule_id` but received {rule_id!r}")
        return await self._get(
            path_template("/accounts/{account_id}/gateway/rules/{rule_id}", account_id=account_id, rule_id=rule_id),
            options=make_request_options(
                extra_headers=extra_headers,
                extra_query=extra_query,
                extra_body=extra_body,
                timeout=timeout,
                post_parser=ResultWrapper[Optional[GatewayRule]]._unwrapper,
            ),
            cast_to=cast(Type[Optional[GatewayRule]], ResultWrapper[GatewayRule]),
        )

    def list_tenant(
        self,
        *,
        account_id: str,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> AsyncPaginator[GatewayRule, AsyncSinglePage[GatewayRule]]:
        """
        List Zero Trust Gateway rules for the parent account of an account in the MSP
        configuration.

        Args:
          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not account_id:
            raise ValueError(f"Expected a non-empty value for `account_id` but received {account_id!r}")
        return self._get_api_list(
            path_template("/accounts/{account_id}/gateway/rules/tenant", account_id=account_id),
            page=AsyncSinglePage[GatewayRule],
            options=make_request_options(
                extra_headers=extra_headers, extra_query=extra_query, extra_body=extra_body, timeout=timeout
            ),
            model=GatewayRule,
        )

    async def reset_expiration(
        self,
        rule_id: str,
        *,
        account_id: str,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> Optional[GatewayRule]:
        """
        Resets the expiration of a Zero Trust Gateway Rule if its duration elapsed and
        it has a default duration. The Zero Trust Gateway Rule must have values for both
        `expiration.expires_at` and `expiration.duration`.

        Args:
          rule_id: Identify the API resource with a UUID.

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not account_id:
            raise ValueError(f"Expected a non-empty value for `account_id` but received {account_id!r}")
        if not rule_id:
            raise ValueError(f"Expected a non-empty value for `rule_id` but received {rule_id!r}")
        return await self._post(
            path_template(
                "/accounts/{account_id}/gateway/rules/{rule_id}/reset_expiration",
                account_id=account_id,
                rule_id=rule_id,
            ),
            options=make_request_options(
                extra_headers=extra_headers,
                extra_query=extra_query,
                extra_body=extra_body,
                timeout=timeout,
                post_parser=ResultWrapper[Optional[GatewayRule]]._unwrapper,
            ),
            cast_to=cast(Type[Optional[GatewayRule]], ResultWrapper[GatewayRule]),
        )


class RulesResourceWithRawResponse:
    def __init__(self, rules: RulesResource) -> None:
        self._rules = rules

        self.create = to_raw_response_wrapper(
            rules.create,
        )
        self.update = to_raw_response_wrapper(
            rules.update,
        )
        self.list = to_raw_response_wrapper(
            rules.list,
        )
        self.delete = to_raw_response_wrapper(
            rules.delete,
        )
        self.get = to_raw_response_wrapper(
            rules.get,
        )
        self.list_tenant = to_raw_response_wrapper(
            rules.list_tenant,
        )
        self.reset_expiration = to_raw_response_wrapper(
            rules.reset_expiration,
        )


class AsyncRulesResourceWithRawResponse:
    def __init__(self, rules: AsyncRulesResource) -> None:
        self._rules = rules

        self.create = async_to_raw_response_wrapper(
            rules.create,
        )
        self.update = async_to_raw_response_wrapper(
            rules.update,
        )
        self.list = async_to_raw_response_wrapper(
            rules.list,
        )
        self.delete = async_to_raw_response_wrapper(
            rules.delete,
        )
        self.get = async_to_raw_response_wrapper(
            rules.get,
        )
        self.list_tenant = async_to_raw_response_wrapper(
            rules.list_tenant,
        )
        self.reset_expiration = async_to_raw_response_wrapper(
            rules.reset_expiration,
        )


class RulesResourceWithStreamingResponse:
    def __init__(self, rules: RulesResource) -> None:
        self._rules = rules

        self.create = to_streamed_response_wrapper(
            rules.create,
        )
        self.update = to_streamed_response_wrapper(
            rules.update,
        )
        self.list = to_streamed_response_wrapper(
            rules.list,
        )
        self.delete = to_streamed_response_wrapper(
            rules.delete,
        )
        self.get = to_streamed_response_wrapper(
            rules.get,
        )
        self.list_tenant = to_streamed_response_wrapper(
            rules.list_tenant,
        )
        self.reset_expiration = to_streamed_response_wrapper(
            rules.reset_expiration,
        )


class AsyncRulesResourceWithStreamingResponse:
    def __init__(self, rules: AsyncRulesResource) -> None:
        self._rules = rules

        self.create = async_to_streamed_response_wrapper(
            rules.create,
        )
        self.update = async_to_streamed_response_wrapper(
            rules.update,
        )
        self.list = async_to_streamed_response_wrapper(
            rules.list,
        )
        self.delete = async_to_streamed_response_wrapper(
            rules.delete,
        )
        self.get = async_to_streamed_response_wrapper(
            rules.get,
        )
        self.list_tenant = async_to_streamed_response_wrapper(
            rules.list_tenant,
        )
        self.reset_expiration = async_to_streamed_response_wrapper(
            rules.reset_expiration,
        )



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from .captions import (
    CaptionsResource,
    AsyncCaptionsResource,
    CaptionsResourceWithRawResponse,
    AsyncCaptionsResourceWithRawResponse,
    CaptionsResourceWithStreamingResponse,
    AsyncCaptionsResourceWithStreamingResponse,
)
from .language import (
    LanguageResource,
    AsyncLanguageResource,
    LanguageResourceWithRawResponse,
    AsyncLanguageResourceWithRawResponse,
    LanguageResourceWithStreamingResponse,
    AsyncLanguageResourceWithStreamingResponse,
)

__all__ = [
    "LanguageResource",
    "AsyncLanguageResource",
    "LanguageResourceWithRawResponse",
    "AsyncLanguageResourceWithRawResponse",
    "LanguageResourceWithStreamingResponse",
    "AsyncLanguageResourceWithStreamingResponse",
    "CaptionsResource",
    "AsyncCaptionsResource",
    "CaptionsResourceWithRawResponse",
    "AsyncCaptionsResourceWithRawResponse",
    "CaptionsResourceWithStreamingResponse",
    "AsyncCaptionsResourceWithStreamingResponse",
]



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from ...._models import BaseModel

__all__ = ["LimitListResponse"]


class LimitListResponse(BaseModel):
    max_custom_regex_entries: int
    """Maximum number of custom regex entries allowed for the account."""

    max_dataset_cells: int
    """
    Maximum number of dataset cells allowed for the account, across all EDM and CWL
    datasets.
    """

    max_document_fingerprints: int
    """Maximum number of document fingerprints allowed for the account."""

    used_custom_regex_entries: int
    """Number of custom regex entries currently configured for the account."""

    used_dataset_cells: int
    """
    Number of dataset cells currently configured for the account, across all EDM and
    CWL datasets. Document fingerprints do not count towards this limit.
    """

    used_document_fingerprints: int
    """Number of document fingerprints currently configured for the account."""



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from __future__ import annotations

from typing import Iterable, Optional
from typing_extensions import Literal

import httpx

from ..._types import Body, Omit, Query, Headers, NotGiven, omit, not_given
from ..._utils import path_template, maybe_transform
from ..._compat import cached_property
from ..._resource import SyncAPIResource, AsyncAPIResource
from ..._response import (
    to_raw_response_wrapper,
    to_streamed_response_wrapper,
    async_to_raw_response_wrapper,
    async_to_streamed_response_wrapper,
)
from ...pagination import SyncSinglePage, AsyncSinglePage, SyncV4PagePagination, AsyncV4PagePagination
from ..._base_client import AsyncPaginator, make_request_options
from ...types.abuse_reports import mitigation_list_params, mitigation_review_params
from ...types.abuse_reports.mitigation_list_response import MitigationListResponse
from ...types.abuse_reports.mitigation_review_response import MitigationReviewResponse

__all__ = ["MitigationsResource", "AsyncMitigationsResource"]


class MitigationsResource(SyncAPIResource):
    @cached_property
    def with_raw_response(self) -> MitigationsResourceWithRawResponse:
        """
        This property can be used as a prefix for any HTTP method call to return
        the raw response object instead of the parsed content.

        For more information, see https://www.github.com/cloudflare/cloudflare-python#accessing-raw-response-data-eg-headers
        """
        return MitigationsResourceWithRawResponse(self)

    @cached_property
    def with_streaming_response(self) -> MitigationsResourceWithStreamingResponse:
        """
        An alternative to `.with_raw_response` that doesn't eagerly read the response body.

        For more information, see https://www.github.com/cloudflare/cloudflare-python#with_streaming_response
        """
        return MitigationsResourceWithStreamingResponse(self)

    def list(
        self,
        report_id: str,
        *,
        account_id: str,
        effective_after: str | Omit = omit,
        effective_before: str | Omit = omit,
        entity_type: Literal["url_pattern", "account", "zone"] | Omit = omit,
        page: int | Omit = omit,
        per_page: int | Omit = omit,
        sort: Literal[
            "type,asc",
            "type,desc",
            "effective_date,asc",
            "effective_date,desc",
            "status,asc",
            "status,desc",
            "entity_type,asc",
            "entity_type,desc",
        ]
        | Omit = omit,
        status: Literal["pending", "active", "in_review", "cancelled", "removed"] | Omit = omit,
        type: Literal[
            "legal_block",
            "misleading_interstitial",
            "phishing_interstitial",
            "network_block",
            "rate_limit_cache",
            "account_suspend",
            "redirect_video_stream",
        ]
        | Omit = omit,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> SyncV4PagePagination[Optional[MitigationListResponse]]:
        """
        List mitigations done to remediate the abuse report.

        Args:
          effective_after: Returns mitigation that were dispatched after the given date

          effective_before: Returns mitigations that were dispatched before the given date

          entity_type: Filter by the type of entity the mitigation impacts.

          page: Where in pagination to start listing abuse reports

          per_page: How many abuse reports per page to list

          sort: A property to sort by, followed by the order

          status: Filter by the status of the mitigation.

          type: Filter by the type of mitigation. This filter parameter can be specified
              multiple times to include multiple types of mitigations in the result set, e.g.
              ?type=rate_limit_cache&type=legal_block.

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not account_id:
            raise ValueError(f"Expected a non-empty value for `account_id` but received {account_id!r}")
        if not report_id:
            raise ValueError(f"Expected a non-empty value for `report_id` but received {report_id!r}")
        return self._get_api_list(
            path_template(
                "/accounts/{account_id}/abuse-reports/{report_id}/mitigations",
                account_id=account_id,
                report_id=report_id,
            ),
            page=SyncV4PagePagination[Optional[MitigationListResponse]],
            options=make_request_options(
                extra_headers=extra_headers,
                extra_query=extra_query,
                extra_body=extra_body,
                timeout=timeout,
                query=maybe_transform(
                    {
                        "effective_after": effective_after,
                        "effective_before": effective_before,
                        "entity_type": entity_type,
                        "page": page,
                        "per_page": per_page,
                        "sort": sort,
                        "status": status,
                        "type": type,
                    },
                    mitigation_list_params.MitigationListParams,
                ),
            ),
            model=MitigationListResponse,
        )

    def review(
        self,
        report_id: str,
        *,
        account_id: str,
        appeals: Iterable[mitigation_review_params.Appeal],
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> SyncSinglePage[MitigationReviewResponse]:
        """
        Request a review for mitigations on an account.

        Args:
          appeals: List of mitigations to appeal.

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not account_id:
            raise ValueError(f"Expected a non-empty value for `account_id` but received {account_id!r}")
        if not report_id:
            raise ValueError(f"Expected a non-empty value for `report_id` but received {report_id!r}")
        return self._get_api_list(
            path_template(
                "/accounts/{account_id}/abuse-reports/{report_id}/mitigations/appeal",
                account_id=account_id,
                report_id=report_id,
            ),
            page=SyncSinglePage[MitigationReviewResponse],
            body=maybe_transform({"appeals": appeals}, mitigation_review_params.MitigationReviewParams),
            options=make_request_options(
                extra_headers=extra_headers, extra_query=extra_query, extra_body=extra_body, timeout=timeout
            ),
            model=MitigationReviewResponse,
            method="post",
        )


class AsyncMitigationsResource(AsyncAPIResource):
    @cached_property
    def with_raw_response(self) -> AsyncMitigationsResourceWithRawResponse:
        """
        This property can be used as a prefix for any HTTP method call to return
        the raw response object instead of the parsed content.

        For more information, see https://www.github.com/cloudflare/cloudflare-python#accessing-raw-response-data-eg-headers
        """
        return AsyncMitigationsResourceWithRawResponse(self)

    @cached_property
    def with_streaming_response(self) -> AsyncMitigationsResourceWithStreamingResponse:
        """
        An alternative to `.with_raw_response` that doesn't eagerly read the response body.

        For more information, see https://www.github.com/cloudflare/cloudflare-python#with_streaming_response
        """
        return AsyncMitigationsResourceWithStreamingResponse(self)

    def list(
        self,
        report_id: str,
        *,
        account_id: str,
        effective_after: str | Omit = omit,
        effective_before: str | Omit = omit,
        entity_type: Literal["url_pattern", "account", "zone"] | Omit = omit,
        page: int | Omit = omit,
        per_page: int | Omit = omit,
        sort: Literal[
            "type,asc",
            "type,desc",
            "effective_date,asc",
            "effective_date,desc",
            "status,asc",
            "status,desc",
            "entity_type,asc",
            "entity_type,desc",
        ]
        | Omit = omit,
        status: Literal["pending", "active", "in_review", "cancelled", "removed"] | Omit = omit,
        type: Literal[
            "legal_block",
            "misleading_interstitial",
            "phishing_interstitial",
            "network_block",
            "rate_limit_cache",
            "account_suspend",
            "redirect_video_stream",
        ]
        | Omit = omit,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> AsyncPaginator[Optional[MitigationListResponse], AsyncV4PagePagination[Optional[MitigationListResponse]]]:
        """
        List mitigations done to remediate the abuse report.

        Args:
          effective_after: Returns mitigation that were dispatched after the given date

          effective_before: Returns mitigations that were dispatched before the given date

          entity_type: Filter by the type of entity the mitigation impacts.

          page: Where in pagination to start listing abuse reports

          per_page: How many abuse reports per page to list

          sort: A property to sort by, followed by the order

          status: Filter by the status of the mitigation.

          type: Filter by the type of mitigation. This filter parameter can be specified
              multiple times to include multiple types of mitigations in the result set, e.g.
              ?type=rate_limit_cache&type=legal_block.

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not account_id:
            raise ValueError(f"Expected a non-empty value for `account_id` but received {account_id!r}")
        if not report_id:
            raise ValueError(f"Expected a non-empty value for `report_id` but received {report_id!r}")
        return self._get_api_list(
            path_template(
                "/accounts/{account_id}/abuse-reports/{report_id}/mitigations",
                account_id=account_id,
                report_id=report_id,
            ),
            page=AsyncV4PagePagination[Optional[MitigationListResponse]],
            options=make_request_options(
                extra_headers=extra_headers,
                extra_query=extra_query,
                extra_body=extra_body,
                timeout=timeout,
                query=maybe_transform(
                    {
                        "effective_after": effective_after,
                        "effective_before": effective_before,
                        "entity_type": entity_type,
                        "page": page,
                        "per_page": per_page,
                        "sort": sort,
                        "status": status,
                        "type": type,
                    },
                    mitigation_list_params.MitigationListParams,
                ),
            ),
            model=MitigationListResponse,
        )

    def review(
        self,
        report_id: str,
        *,
        account_id: str,
        appeals: Iterable[mitigation_review_params.Appeal],
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> AsyncPaginator[MitigationReviewResponse, AsyncSinglePage[MitigationReviewResponse]]:
        """
        Request a review for mitigations on an account.

        Args:
          appeals: List of mitigations to appeal.

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not account_id:
            raise ValueError(f"Expected a non-empty value for `account_id` but received {account_id!r}")
        if not report_id:
            raise ValueError(f"Expected a non-empty value for `report_id` but received {report_id!r}")
        return self._get_api_list(
            path_template(
                "/accounts/{account_id}/abuse-reports/{report_id}/mitigations/appeal",
                account_id=account_id,
                report_id=report_id,
            ),
            page=AsyncSinglePage[MitigationReviewResponse],
            body=maybe_transform({"appeals": appeals}, mitigation_review_params.MitigationReviewParams),
            options=make_request_options(
                extra_headers=extra_headers, extra_query=extra_query, extra_body=extra_body, timeout=timeout
            ),
            model=MitigationReviewResponse,
            method="post",
        )


class MitigationsResourceWithRawResponse:
    def __init__(self, mitigations: MitigationsResource) -> None:
        self._mitigations = mitigations

        self.list = to_raw_response_wrapper(
            mitigations.list,
        )
        self.review = to_raw_response_wrapper(
            mitigations.review,
        )


class AsyncMitigationsResourceWithRawResponse:
    def __init__(self, mitigations: AsyncMitigationsResource) -> None:
        self._mitigations = mitigations

        self.list = async_to_raw_response_wrapper(
            mitigations.list,
        )
        self.review = async_to_raw_response_wrapper(
            mitigations.review,
        )


class MitigationsResourceWithStreamingResponse:
    def __init__(self, mitigations: MitigationsResource) -> None:
        self._mitigations = mitigations

        self.list = to_streamed_response_wrapper(
            mitigations.list,
        )
        self.review = to_streamed_response_wrapper(
            mitigations.review,
        )


class AsyncMitigationsResourceWithStreamingResponse:
    def __init__(self, mitigations: AsyncMitigationsResource) -> None:
        self._mitigations = mitigations

        self.list = async_to_streamed_response_wrapper(
            mitigations.list,
        )
        self.review = async_to_streamed_response_wrapper(
            mitigations.review,
        )



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from __future__ import annotations

from typing import Union
from typing_extensions import Literal, TypeAlias, TypedDict

from ..._types import SequenceNotStr

__all__ = ["EdgeIPsParam", "Dynamic", "Static"]


class Dynamic(TypedDict, total=False):
    connectivity: Literal["all", "ipv4", "ipv6"]
    """The IP versions supported for inbound connections on Spectrum anycast IPs."""

    type: Literal["dynamic"]
    """The type of edge IP configuration specified.

    Dynamically allocated edge IPs use Spectrum anycast IPs in accordance with the
    connectivity you specify. Only valid with CNAME DNS names.
    """


class Static(TypedDict, total=False):
    ips: SequenceNotStr[str]
    """
    The array of customer owned IPs we broadcast via anycast for this hostname and
    application.
    """

    type: Literal["static"]
    """The type of edge IP configuration specified.

    Statically allocated edge IPs use customer IPs in accordance with the ips array
    you specify. Only valid with ADDRESS DNS names.
    """


EdgeIPsParam: TypeAlias = Union[Dynamic, Static]



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from __future__ import annotations

from typing import Type, Optional, cast
from typing_extensions import Literal, overload

import httpx

from ...._types import Body, Omit, Query, Headers, NotGiven, SequenceNotStr, omit, not_given
from ...._utils import path_template, required_args, maybe_transform, async_maybe_transform
from ...._compat import cached_property
from ...._resource import SyncAPIResource, AsyncAPIResource
from ...._response import (
    to_raw_response_wrapper,
    to_streamed_response_wrapper,
    async_to_raw_response_wrapper,
    async_to_streamed_response_wrapper,
)
from ...._wrappers import ResultWrapper
from ...._base_client import make_request_options
from ....types.r2.buckets import Provider
from ....types.r2.super_slurper import connectivity_precheck_source_params, connectivity_precheck_target_params
from ....types.r2.buckets.provider import Provider
from ....types.r2.super_slurper.connectivity_precheck_source_response import ConnectivityPrecheckSourceResponse
from ....types.r2.super_slurper.connectivity_precheck_target_response import ConnectivityPrecheckTargetResponse

__all__ = ["ConnectivityPrecheckResource", "AsyncConnectivityPrecheckResource"]


class ConnectivityPrecheckResource(SyncAPIResource):
    @cached_property
    def with_raw_response(self) -> ConnectivityPrecheckResourceWithRawResponse:
        """
        This property can be used as a prefix for any HTTP method call to return
        the raw response object instead of the parsed content.

        For more information, see https://www.github.com/cloudflare/cloudflare-python#accessing-raw-response-data-eg-headers
        """
        return ConnectivityPrecheckResourceWithRawResponse(self)

    @cached_property
    def with_streaming_response(self) -> ConnectivityPrecheckResourceWithStreamingResponse:
        """
        An alternative to `.with_raw_response` that doesn't eagerly read the response body.

        For more information, see https://www.github.com/cloudflare/cloudflare-python#with_streaming_response
        """
        return ConnectivityPrecheckResourceWithStreamingResponse(self)

    @overload
    def source(
        self,
        *,
        account_id: str,
        bucket: str,
        secret: connectivity_precheck_source_params.R2SlurperS3SourceSchemaSecret,
        vendor: Literal["s3"],
        endpoint: Optional[str] | Omit = omit,
        keys: Optional[SequenceNotStr[str]] | Omit = omit,
        path_prefix: Optional[str] | Omit = omit,
        region: Optional[str] | Omit = omit,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> Optional[ConnectivityPrecheckSourceResponse]:
        """
        Check whether tokens are valid against the source bucket

        Args:
          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        ...

    @overload
    def source(
        self,
        *,
        account_id: str,
        bucket: str,
        secret: connectivity_precheck_source_params.R2SlurperGcsSourceSchemaSecret,
        vendor: Literal["gcs"],
        keys: Optional[SequenceNotStr[str]] | Omit = omit,
        path_prefix: Optional[str] | Omit = omit,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> Optional[ConnectivityPrecheckSourceResponse]:
        """
        Check whether tokens are valid against the source bucket

        Args:
          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        ...

    @overload
    def source(
        self,
        *,
        account_id: str,
        bucket: str,
        secret: connectivity_precheck_source_params.R2SlurperR2SourceSchemaSecret,
        vendor: Provider,
        jurisdiction: Literal["default", "eu", "fedramp"] | Omit = omit,
        keys: Optional[SequenceNotStr[str]] | Omit = omit,
        path_prefix: Optional[str] | Omit = omit,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> Optional[ConnectivityPrecheckSourceResponse]:
        """
        Check whether tokens are valid against the source bucket

        Args:
          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        ...

    @required_args(["account_id", "bucket", "secret", "vendor"])
    def source(
        self,
        *,
        account_id: str,
        bucket: str,
        secret: connectivity_precheck_source_params.R2SlurperS3SourceSchemaSecret
        | connectivity_precheck_source_params.R2SlurperGcsSourceSchemaSecret
        | connectivity_precheck_source_params.R2SlurperR2SourceSchemaSecret,
        vendor: Literal["s3"] | Literal["gcs"] | Provider,
        endpoint: Optional[str] | Omit = omit,
        keys: Optional[SequenceNotStr[str]] | Omit = omit,
        path_prefix: Optional[str] | Omit = omit,
        region: Optional[str] | Omit = omit,
        jurisdiction: Literal["default", "eu", "fedramp"] | Omit = omit,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> Optional[ConnectivityPrecheckSourceResponse]:
        if not account_id:
            raise ValueError(f"Expected a non-empty value for `account_id` but received {account_id!r}")
        return self._put(
            path_template("/accounts/{account_id}/slurper/source/connectivity-precheck", account_id=account_id),
            body=maybe_transform(
                {
                    "bucket": bucket,
                    "secret": secret,
                    "vendor": vendor,
                    "endpoint": endpoint,
                    "keys": keys,
                    "path_prefix": path_prefix,
                    "region": region,
                    "jurisdiction": jurisdiction,
                },
                connectivity_precheck_source_params.ConnectivityPrecheckSourceParams,
            ),
            options=make_request_options(
                extra_headers=extra_headers,
                extra_query=extra_query,
                extra_body=extra_body,
                timeout=timeout,
                post_parser=ResultWrapper[Optional[ConnectivityPrecheckSourceResponse]]._unwrapper,
            ),
            cast_to=cast(
                Type[Optional[ConnectivityPrecheckSourceResponse]], ResultWrapper[ConnectivityPrecheckSourceResponse]
            ),
        )

    def target(
        self,
        *,
        account_id: str,
        bucket: str,
        secret: connectivity_precheck_target_params.Secret,
        vendor: Provider,
        jurisdiction: Literal["default", "eu", "fedramp"] | Omit = omit,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> Optional[ConnectivityPrecheckTargetResponse]:
        """
        Check whether tokens are valid against the target bucket

        Args:
          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not account_id:
            raise ValueError(f"Expected a non-empty value for `account_id` but received {account_id!r}")
        return self._put(
            path_template("/accounts/{account_id}/slurper/target/connectivity-precheck", account_id=account_id),
            body=maybe_transform(
                {
                    "bucket": bucket,
                    "secret": secret,
                    "vendor": vendor,
                    "jurisdiction": jurisdiction,
                },
                connectivity_precheck_target_params.ConnectivityPrecheckTargetParams,
            ),
            options=make_request_options(
                extra_headers=extra_headers,
                extra_query=extra_query,
                extra_body=extra_body,
                timeout=timeout,
                post_parser=ResultWrapper[Optional[ConnectivityPrecheckTargetResponse]]._unwrapper,
            ),
            cast_to=cast(
                Type[Optional[ConnectivityPrecheckTargetResponse]], ResultWrapper[ConnectivityPrecheckTargetResponse]
            ),
        )


class AsyncConnectivityPrecheckResource(AsyncAPIResource):
    @cached_property
    def with_raw_response(self) -> AsyncConnectivityPrecheckResourceWithRawResponse:
        """
        This property can be used as a prefix for any HTTP method call to return
        the raw response object instead of the parsed content.

        For more information, see https://www.github.com/cloudflare/cloudflare-python#accessing-raw-response-data-eg-headers
        """
        return AsyncConnectivityPrecheckResourceWithRawResponse(self)

    @cached_property
    def with_streaming_response(self) -> AsyncConnectivityPrecheckResourceWithStreamingResponse:
        """
        An alternative to `.with_raw_response` that doesn't eagerly read the response body.

        For more information, see https://www.github.com/cloudflare/cloudflare-python#with_streaming_response
        """
        return AsyncConnectivityPrecheckResourceWithStreamingResponse(self)

    @overload
    async def source(
        self,
        *,
        account_id: str,
        bucket: str,
        secret: connectivity_precheck_source_params.R2SlurperS3SourceSchemaSecret,
        vendor: Literal["s3"],
        endpoint: Optional[str] | Omit = omit,
        keys: Optional[SequenceNotStr[str]] | Omit = omit,
        path_prefix: Optional[str] | Omit = omit,
        region: Optional[str] | Omit = omit,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> Optional[ConnectivityPrecheckSourceResponse]:
        """
        Check whether tokens are valid against the source bucket

        Args:
          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        ...

    @overload
    async def source(
        self,
        *,
        account_id: str,
        bucket: str,
        secret: connectivity_precheck_source_params.R2SlurperGcsSourceSchemaSecret,
        vendor: Literal["gcs"],
        keys: Optional[SequenceNotStr[str]] | Omit = omit,
        path_prefix: Optional[str] | Omit = omit,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> Optional[ConnectivityPrecheckSourceResponse]:
        """
        Check whether tokens are valid against the source bucket

        Args:
          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        ...

    @overload
    async def source(
        self,
        *,
        account_id: str,
        bucket: str,
        secret: connectivity_precheck_source_params.R2SlurperR2SourceSchemaSecret,
        vendor: Provider,
        jurisdiction: Literal["default", "eu", "fedramp"] | Omit = omit,
        keys: Optional[SequenceNotStr[str]] | Omit = omit,
        path_prefix: Optional[str] | Omit = omit,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> Optional[ConnectivityPrecheckSourceResponse]:
        """
        Check whether tokens are valid against the source bucket

        Args:
          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        ...

    @required_args(["account_id", "bucket", "secret", "vendor"])
    async def source(
        self,
        *,
        account_id: str,
        bucket: str,
        secret: connectivity_precheck_source_params.R2SlurperS3SourceSchemaSecret
        | connectivity_precheck_source_params.R2SlurperGcsSourceSchemaSecret
        | connectivity_precheck_source_params.R2SlurperR2SourceSchemaSecret,
        vendor: Literal["s3"] | Literal["gcs"] | Provider,
        endpoint: Optional[str] | Omit = omit,
        keys: Optional[SequenceNotStr[str]] | Omit = omit,
        path_prefix: Optional[str] | Omit = omit,
        region: Optional[str] | Omit = omit,
        jurisdiction: Literal["default", "eu", "fedramp"] | Omit = omit,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> Optional[ConnectivityPrecheckSourceResponse]:
        if not account_id:
            raise ValueError(f"Expected a non-empty value for `account_id` but received {account_id!r}")
        return await self._put(
            path_template("/accounts/{account_id}/slurper/source/connectivity-precheck", account_id=account_id),
            body=await async_maybe_transform(
                {
                    "bucket": bucket,
                    "secret": secret,
                    "vendor": vendor,
                    "endpoint": endpoint,
                    "keys": keys,
                    "path_prefix": path_prefix,
                    "region": region,
                    "jurisdiction": jurisdiction,
                },
                connectivity_precheck_source_params.ConnectivityPrecheckSourceParams,
            ),
            options=make_request_options(
                extra_headers=extra_headers,
                extra_query=extra_query,
                extra_body=extra_body,
                timeout=timeout,
                post_parser=ResultWrapper[Optional[ConnectivityPrecheckSourceResponse]]._unwrapper,
            ),
            cast_to=cast(
                Type[Optional[ConnectivityPrecheckSourceResponse]], ResultWrapper[ConnectivityPrecheckSourceResponse]
            ),
        )

    async def target(
        self,
        *,
        account_id: str,
        bucket: str,
        secret: connectivity_precheck_target_params.Secret,
        vendor: Provider,
        jurisdiction: Literal["default", "eu", "fedramp"] | Omit = omit,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> Optional[ConnectivityPrecheckTargetResponse]:
        """
        Check whether tokens are valid against the target bucket

        Args:
          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not account_id:
            raise ValueError(f"Expected a non-empty value for `account_id` but received {account_id!r}")
        return await self._put(
            path_template("/accounts/{account_id}/slurper/target/connectivity-precheck", account_id=account_id),
            body=await async_maybe_transform(
                {
                    "bucket": bucket,
                    "secret": secret,
                    "vendor": vendor,
                    "jurisdiction": jurisdiction,
                },
                connectivity_precheck_target_params.ConnectivityPrecheckTargetParams,
            ),
            options=make_request_options(
                extra_headers=extra_headers,
                extra_query=extra_query,
                extra_body=extra_body,
                timeout=timeout,
                post_parser=ResultWrapper[Optional[ConnectivityPrecheckTargetResponse]]._unwrapper,
            ),
            cast_to=cast(
                Type[Optional[ConnectivityPrecheckTargetResponse]], ResultWrapper[ConnectivityPrecheckTargetResponse]
            ),
        )


class ConnectivityPrecheckResourceWithRawResponse:
    def __init__(self, connectivity_precheck: ConnectivityPrecheckResource) -> None:
        self._connectivity_precheck = connectivity_precheck

        self.source = to_raw_response_wrapper(
            connectivity_precheck.source,
        )
        self.target = to_raw_response_wrapper(
            connectivity_precheck.target,
        )


class AsyncConnectivityPrecheckResourceWithRawResponse:
    def __init__(self, connectivity_precheck: AsyncConnectivityPrecheckResource) -> None:
        self._connectivity_precheck = connectivity_precheck

        self.source = async_to_raw_response_wrapper(
            connectivity_precheck.source,
        )
        self.target = async_to_raw_response_wrapper(
            connectivity_precheck.target,
        )


class ConnectivityPrecheckResourceWithStreamingResponse:
    def __init__(self, connectivity_precheck: ConnectivityPrecheckResource) -> None:
        self._connectivity_precheck = connectivity_precheck

        self.source = to_streamed_response_wrapper(
            connectivity_precheck.source,
        )
        self.target = to_streamed_response_wrapper(
            connectivity_precheck.target,
        )


class AsyncConnectivityPrecheckResourceWithStreamingResponse:
    def __init__(self, connectivity_precheck: AsyncConnectivityPrecheckResource) -> None:
        self._connectivity_precheck = connectivity_precheck

        self.source = async_to_streamed_response_wrapper(
            connectivity_precheck.source,
        )
        self.target = async_to_streamed_response_wrapper(
            connectivity_precheck.target,
        )



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from __future__ import annotations

from typing_extensions import Literal, Required, TypedDict

__all__ = ["OperationEditParams"]


class OperationEditParams(TypedDict, total=False):
    zone_id: Required[str]
    """Identifier."""

    state: Literal["review", "ignored"]
    """Mark state of operation in API Discovery

    - `review` - Mark operation as for review
    - `ignored` - Mark operation as ignored
    """



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from .fallback_origin import (
    FallbackOriginResource,
    AsyncFallbackOriginResource,
    FallbackOriginResourceWithRawResponse,
    AsyncFallbackOriginResourceWithRawResponse,
    FallbackOriginResourceWithStreamingResponse,
    AsyncFallbackOriginResourceWithStreamingResponse,
)
from .certificate_pack import (
    CertificatePackResource,
    AsyncCertificatePackResource,
    CertificatePackResourceWithRawResponse,
    AsyncCertificatePackResourceWithRawResponse,
    CertificatePackResourceWithStreamingResponse,
    AsyncCertificatePackResourceWithStreamingResponse,
)
from .custom_hostnames import (
    CustomHostnamesResource,
    AsyncCustomHostnamesResource,
    CustomHostnamesResourceWithRawResponse,
    AsyncCustomHostnamesResourceWithRawResponse,
    CustomHostnamesResourceWithStreamingResponse,
    AsyncCustomHostnamesResourceWithStreamingResponse,
)

__all__ = [
    "FallbackOriginResource",
    "AsyncFallbackOriginResource",
    "FallbackOriginResourceWithRawResponse",
    "AsyncFallbackOriginResourceWithRawResponse",
    "FallbackOriginResourceWithStreamingResponse",
    "AsyncFallbackOriginResourceWithStreamingResponse",
    "CertificatePackResource",
    "AsyncCertificatePackResource",
    "CertificatePackResourceWithRawResponse",
    "AsyncCertificatePackResourceWithRawResponse",
    "CertificatePackResourceWithStreamingResponse",
    "AsyncCertificatePackResourceWithStreamingResponse",
    "CustomHostnamesResource",
    "AsyncCustomHostnamesResource",
    "CustomHostnamesResourceWithRawResponse",
    "AsyncCustomHostnamesResourceWithRawResponse",
    "CustomHostnamesResourceWithStreamingResponse",
    "AsyncCustomHostnamesResourceWithStreamingResponse",
]



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from __future__ import annotations

from typing import List
from typing_extensions import Literal, Required, TypedDict

from ..._types import SequenceNotStr

__all__ = ["ResourceTaggingListParams"]


class ResourceTaggingListParams(TypedDict, total=False):
    account_id: Required[str]
    """Identifier."""

    cursor: str
    """Cursor for pagination."""

    tag: SequenceNotStr[str]
    """Filter resources by tag criteria.

    This parameter can be repeated multiple times, with AND logic between
    parameters.

    Supported syntax:

    - **Key-only**: `tag=<key>` - Resource must have the tag key (e.g.,
      `tag=production`)
    - **Key-value**: `tag=<key>=<value>` - Resource must have the tag with specific
      value (e.g., `tag=env=prod`)
    - **Multiple values (OR)**: `tag=<key>=<v1>,<v2>` - Resource must have tag with
      any of the values (e.g., `tag=env=prod,staging`)
    - **Negate key-only**: `tag=!<key>` - Resource must not have the tag key (e.g.,
      `tag=!archived`)
    - **Negate key-value**: `tag=<key>!=<value>` - Resource must not have the tag
      with specific value (e.g., `tag=region!=us-west-1`)

    Multiple tag parameters are combined with AND logic.
    """

    type: List[
        Literal[
            "access_application",
            "access_application_policy",
            "access_group",
            "account",
            "ai_gateway",
            "alerting_policy",
            "alerting_webhook",
            "api_gateway_operation",
            "cloudflared_tunnel",
            "custom_certificate",
            "custom_hostname",
            "d1_database",
            "dns_record",
            "durable_object_namespace",
            "gateway_list",
            "gateway_rule",
            "image",
            "kv_namespace",
            "managed_client_certificate",
            "queue",
            "r2_bucket",
            "resource_share",
            "stream_live_input",
            "stream_video",
            "worker",
            "worker_version",
            "zone",
        ]
    ]
    """Filter by resource type.

    Can be repeated to filter by multiple types (OR logic). Example:
    ?type=zone&type=worker
    """



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from typing import Optional
from datetime import datetime

from pydantic import Field as FieldInfo

from ....._models import BaseModel

__all__ = ["AccessRequest"]


class AccessRequest(BaseModel):
    action: Optional[str] = None
    """The event that occurred, such as a login attempt."""

    allowed: Optional[bool] = None
    """The result of the authentication event."""

    app_domain: Optional[str] = None
    """The URL of the Access application."""

    app_uid: Optional[str] = None
    """The unique identifier for the Access application."""

    connection: Optional[str] = None
    """The IdP used to authenticate."""

    created_at: Optional[datetime] = None

    ip_address: Optional[str] = None
    """The IP address of the authenticating user."""

    rayid: Optional[str] = FieldInfo(alias="ray_id", default=None)
    """The unique identifier for the request to Cloudflare."""

    user_email: Optional[str] = None
    """The email address of the authenticating user."""



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from __future__ import annotations

from typing_extensions import Literal

import httpx

from .warp import (
    WARPResource,
    AsyncWARPResource,
    WARPResourceWithRawResponse,
    AsyncWARPResourceWithRawResponse,
    WARPResourceWithStreamingResponse,
    AsyncWARPResourceWithStreamingResponse,
)
from ....._types import Body, Omit, Query, Headers, NotGiven, omit, not_given
from ....._utils import path_template, maybe_transform
from ....._compat import cached_property
from ....._resource import SyncAPIResource, AsyncAPIResource
from ....._response import (
    to_raw_response_wrapper,
    to_streamed_response_wrapper,
    async_to_raw_response_wrapper,
    async_to_streamed_response_wrapper,
)
from .....pagination import SyncV4PagePaginationArray, AsyncV4PagePaginationArray
from ....._base_client import AsyncPaginator, make_request_options
from .cloudflare_source import (
    CloudflareSourceResource,
    AsyncCloudflareSourceResource,
    CloudflareSourceResourceWithRawResponse,
    AsyncCloudflareSourceResourceWithRawResponse,
    CloudflareSourceResourceWithStreamingResponse,
    AsyncCloudflareSourceResourceWithStreamingResponse,
)
from .....types.zero_trust.networks import subnet_list_params
from .....types.zero_trust.networks.subnets.subnet import Subnet

__all__ = ["SubnetsResource", "AsyncSubnetsResource"]


class SubnetsResource(SyncAPIResource):
    @cached_property
    def warp(self) -> WARPResource:
        return WARPResource(self._client)

    @cached_property
    def cloudflare_source(self) -> CloudflareSourceResource:
        return CloudflareSourceResource(self._client)

    @cached_property
    def with_raw_response(self) -> SubnetsResourceWithRawResponse:
        """
        This property can be used as a prefix for any HTTP method call to return
        the raw response object instead of the parsed content.

        For more information, see https://www.github.com/cloudflare/cloudflare-python#accessing-raw-response-data-eg-headers
        """
        return SubnetsResourceWithRawResponse(self)

    @cached_property
    def with_streaming_response(self) -> SubnetsResourceWithStreamingResponse:
        """
        An alternative to `.with_raw_response` that doesn't eagerly read the response body.

        For more information, see https://www.github.com/cloudflare/cloudflare-python#with_streaming_response
        """
        return SubnetsResourceWithStreamingResponse(self)

    def list(
        self,
        *,
        account_id: str,
        address_family: Literal["v4", "v6"] | Omit = omit,
        comment: str | Omit = omit,
        existed_at: str | Omit = omit,
        is_default_network: bool | Omit = omit,
        is_deleted: bool | Omit = omit,
        name: str | Omit = omit,
        network: str | Omit = omit,
        page: float | Omit = omit,
        per_page: float | Omit = omit,
        sort_order: Literal["asc", "desc"] | Omit = omit,
        subnet_types: Literal["cloudflare_source", "warp"] | Omit = omit,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> SyncV4PagePaginationArray[Subnet]:
        """
        Lists and filters subnets in an account.

        Args:
          account_id: Cloudflare account ID

          address_family: If set, only include subnets in the given address family - `v4` or `v6`

          comment: If set, only list subnets with the given comment.

          existed_at: If provided, include only resources that were created (and not deleted) before
              this time. URL encoded.

          is_default_network: If `true`, only include default subnets. If `false`, exclude default subnets
              subnets. If not set, all subnets will be included.

          is_deleted: If `true`, only include deleted subnets. If `false`, exclude deleted subnets. If
              not set, all subnets will be included.

          name: If set, only list subnets with the given name

          network: If set, only list the subnet whose network exactly matches the given CIDR.

          page: Page number of paginated results.

          per_page: Number of results to display.

          sort_order: Sort order of the results. `asc` means oldest to newest, `desc` means newest to
              oldest. If not set, they will not be in any particular order.

          subnet_types: If set, the types of subnets to include, separated by comma.

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not account_id:
            raise ValueError(f"Expected a non-empty value for `account_id` but received {account_id!r}")
        return self._get_api_list(
            path_template("/accounts/{account_id}/zerotrust/subnets", account_id=account_id),
            page=SyncV4PagePaginationArray[Subnet],
            options=make_request_options(
                extra_headers=extra_headers,
                extra_query=extra_query,
                extra_body=extra_body,
                timeout=timeout,
                query=maybe_transform(
                    {
                        "address_family": address_family,
                        "comment": comment,
                        "existed_at": existed_at,
                        "is_default_network": is_default_network,
                        "is_deleted": is_deleted,
                        "name": name,
                        "network": network,
                        "page": page,
                        "per_page": per_page,
                        "sort_order": sort_order,
                        "subnet_types": subnet_types,
                    },
                    subnet_list_params.SubnetListParams,
                ),
            ),
            model=Subnet,
        )


class AsyncSubnetsResource(AsyncAPIResource):
    @cached_property
    def warp(self) -> AsyncWARPResource:
        return AsyncWARPResource(self._client)

    @cached_property
    def cloudflare_source(self) -> AsyncCloudflareSourceResource:
        return AsyncCloudflareSourceResource(self._client)

    @cached_property
    def with_raw_response(self) -> AsyncSubnetsResourceWithRawResponse:
        """
        This property can be used as a prefix for any HTTP method call to return
        the raw response object instead of the parsed content.

        For more information, see https://www.github.com/cloudflare/cloudflare-python#accessing-raw-response-data-eg-headers
        """
        return AsyncSubnetsResourceWithRawResponse(self)

    @cached_property
    def with_streaming_response(self) -> AsyncSubnetsResourceWithStreamingResponse:
        """
        An alternative to `.with_raw_response` that doesn't eagerly read the response body.

        For more information, see https://www.github.com/cloudflare/cloudflare-python#with_streaming_response
        """
        return AsyncSubnetsResourceWithStreamingResponse(self)

    def list(
        self,
        *,
        account_id: str,
        address_family: Literal["v4", "v6"] | Omit = omit,
        comment: str | Omit = omit,
        existed_at: str | Omit = omit,
        is_default_network: bool | Omit = omit,
        is_deleted: bool | Omit = omit,
        name: str | Omit = omit,
        network: str | Omit = omit,
        page: float | Omit = omit,
        per_page: float | Omit = omit,
        sort_order: Literal["asc", "desc"] | Omit = omit,
        subnet_types: Literal["cloudflare_source", "warp"] | Omit = omit,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> AsyncPaginator[Subnet, AsyncV4PagePaginationArray[Subnet]]:
        """
        Lists and filters subnets in an account.

        Args:
          account_id: Cloudflare account ID

          address_family: If set, only include subnets in the given address family - `v4` or `v6`

          comment: If set, only list subnets with the given comment.

          existed_at: If provided, include only resources that were created (and not deleted) before
              this time. URL encoded.

          is_default_network: If `true`, only include default subnets. If `false`, exclude default subnets
              subnets. If not set, all subnets will be included.

          is_deleted: If `true`, only include deleted subnets. If `false`, exclude deleted subnets. If
              not set, all subnets will be included.

          name: If set, only list subnets with the given name

          network: If set, only list the subnet whose network exactly matches the given CIDR.

          page: Page number of paginated results.

          per_page: Number of results to display.

          sort_order: Sort order of the results. `asc` means oldest to newest, `desc` means newest to
              oldest. If not set, they will not be in any particular order.

          subnet_types: If set, the types of subnets to include, separated by comma.

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not account_id:
            raise ValueError(f"Expected a non-empty value for `account_id` but received {account_id!r}")
        return self._get_api_list(
            path_template("/accounts/{account_id}/zerotrust/subnets", account_id=account_id),
            page=AsyncV4PagePaginationArray[Subnet],
            options=make_request_options(
                extra_headers=extra_headers,
                extra_query=extra_query,
                extra_body=extra_body,
                timeout=timeout,
                query=maybe_transform(
                    {
                        "address_family": address_family,
                        "comment": comment,
                        "existed_at": existed_at,
                        "is_default_network": is_default_network,
                        "is_deleted": is_deleted,
                        "name": name,
                        "network": network,
                        "page": page,
                        "per_page": per_page,
                        "sort_order": sort_order,
                        "subnet_types": subnet_types,
                    },
                    subnet_list_params.SubnetListParams,
                ),
            ),
            model=Subnet,
        )


class SubnetsResourceWithRawResponse:
    def __init__(self, subnets: SubnetsResource) -> None:
        self._subnets = subnets

        self.list = to_raw_response_wrapper(
            subnets.list,
        )

    @cached_property
    def warp(self) -> WARPResourceWithRawResponse:
        return WARPResourceWithRawResponse(self._subnets.warp)

    @cached_property
    def cloudflare_source(self) -> CloudflareSourceResourceWithRawResponse:
        return CloudflareSourceResourceWithRawResponse(self._subnets.cloudflare_source)


class AsyncSubnetsResourceWithRawResponse:
    def __init__(self, subnets: AsyncSubnetsResource) -> None:
        self._subnets = subnets

        self.list = async_to_raw_response_wrapper(
            subnets.list,
        )

    @cached_property
    def warp(self) -> AsyncWARPResourceWithRawResponse:
        return AsyncWARPResourceWithRawResponse(self._subnets.warp)

    @cached_property
    def cloudflare_source(self) -> AsyncCloudflareSourceResourceWithRawResponse:
        return AsyncCloudflareSourceResourceWithRawResponse(self._subnets.cloudflare_source)


class SubnetsResourceWithStreamingResponse:
    def __init__(self, subnets: SubnetsResource) -> None:
        self._subnets = subnets

        self.list = to_streamed_response_wrapper(
            subnets.list,
        )

    @cached_property
    def warp(self) -> WARPResourceWithStreamingResponse:
        return WARPResourceWithStreamingResponse(self._subnets.warp)

    @cached_property
    def cloudflare_source(self) -> CloudflareSourceResourceWithStreamingResponse:
        return CloudflareSourceResourceWithStreamingResponse(self._subnets.cloudflare_source)


class AsyncSubnetsResourceWithStreamingResponse:
    def __init__(self, subnets: AsyncSubnetsResource) -> None:
        self._subnets = subnets

        self.list = async_to_streamed_response_wrapper(
            subnets.list,
        )

    @cached_property
    def warp(self) -> AsyncWARPResourceWithStreamingResponse:
        return AsyncWARPResourceWithStreamingResponse(self._subnets.warp)

    @cached_property
    def cloudflare_source(self) -> AsyncCloudflareSourceResourceWithStreamingResponse:
        return AsyncCloudflareSourceResourceWithStreamingResponse(self._subnets.cloudflare_source)



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from __future__ import annotations

from typing import Type, Optional, cast

import httpx

from ...._types import Body, Omit, Query, Headers, NotGiven, omit, not_given
from ...._utils import path_template, maybe_transform, async_maybe_transform
from ...._compat import cached_property
from ...._resource import SyncAPIResource, AsyncAPIResource
from ...._response import (
    to_raw_response_wrapper,
    to_streamed_response_wrapper,
    async_to_raw_response_wrapper,
    async_to_streamed_response_wrapper,
)
from ...._wrappers import ResultWrapper
from ...._base_client import make_request_options
from ....types.zero_trust.gateway import logging_update_params
from ....types.zero_trust.gateway.logging_setting import LoggingSetting

__all__ = ["LoggingResource", "AsyncLoggingResource"]


class LoggingResource(SyncAPIResource):
    @cached_property
    def with_raw_response(self) -> LoggingResourceWithRawResponse:
        """
        This property can be used as a prefix for any HTTP method call to return
        the raw response object instead of the parsed content.

        For more information, see https://www.github.com/cloudflare/cloudflare-python#accessing-raw-response-data-eg-headers
        """
        return LoggingResourceWithRawResponse(self)

    @cached_property
    def with_streaming_response(self) -> LoggingResourceWithStreamingResponse:
        """
        An alternative to `.with_raw_response` that doesn't eagerly read the response body.

        For more information, see https://www.github.com/cloudflare/cloudflare-python#with_streaming_response
        """
        return LoggingResourceWithStreamingResponse(self)

    def update(
        self,
        *,
        account_id: str,
        redact_pii: bool | Omit = omit,
        settings_by_rule_type: logging_update_params.SettingsByRuleType | Omit = omit,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> Optional[LoggingSetting]:
        """
        Update logging settings for the current Zero Trust account.

        Args:
          redact_pii: Indicate whether to redact personally identifiable information from activity
              logging (PII fields include source IP, user email, user ID, device ID, URL,
              referrer, and user agent).

          settings_by_rule_type: Configure logging settings for each rule type.

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not account_id:
            raise ValueError(f"Expected a non-empty value for `account_id` but received {account_id!r}")
        return self._put(
            path_template("/accounts/{account_id}/gateway/logging", account_id=account_id),
            body=maybe_transform(
                {
                    "redact_pii": redact_pii,
                    "settings_by_rule_type": settings_by_rule_type,
                },
                logging_update_params.LoggingUpdateParams,
            ),
            options=make_request_options(
                extra_headers=extra_headers,
                extra_query=extra_query,
                extra_body=extra_body,
                timeout=timeout,
                post_parser=ResultWrapper[Optional[LoggingSetting]]._unwrapper,
            ),
            cast_to=cast(Type[Optional[LoggingSetting]], ResultWrapper[LoggingSetting]),
        )

    def get(
        self,
        *,
        account_id: str,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> Optional[LoggingSetting]:
        """
        Retrieve the current logging settings for the Zero Trust account.

        Args:
          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not account_id:
            raise ValueError(f"Expected a non-empty value for `account_id` but received {account_id!r}")
        return self._get(
            path_template("/accounts/{account_id}/gateway/logging", account_id=account_id),
            options=make_request_options(
                extra_headers=extra_headers,
                extra_query=extra_query,
                extra_body=extra_body,
                timeout=timeout,
                post_parser=ResultWrapper[Optional[LoggingSetting]]._unwrapper,
            ),
            cast_to=cast(Type[Optional[LoggingSetting]], ResultWrapper[LoggingSetting]),
        )


class AsyncLoggingResource(AsyncAPIResource):
    @cached_property
    def with_raw_response(self) -> AsyncLoggingResourceWithRawResponse:
        """
        This property can be used as a prefix for any HTTP method call to return
        the raw response object instead of the parsed content.

        For more information, see https://www.github.com/cloudflare/cloudflare-python#accessing-raw-response-data-eg-headers
        """
        return AsyncLoggingResourceWithRawResponse(self)

    @cached_property
    def with_streaming_response(self) -> AsyncLoggingResourceWithStreamingResponse:
        """
        An alternative to `.with_raw_response` that doesn't eagerly read the response body.

        For more information, see https://www.github.com/cloudflare/cloudflare-python#with_streaming_response
        """
        return AsyncLoggingResourceWithStreamingResponse(self)

    async def update(
        self,
        *,
        account_id: str,
        redact_pii: bool | Omit = omit,
        settings_by_rule_type: logging_update_params.SettingsByRuleType | Omit = omit,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> Optional[LoggingSetting]:
        """
        Update logging settings for the current Zero Trust account.

        Args:
          redact_pii: Indicate whether to redact personally identifiable information from activity
              logging (PII fields include source IP, user email, user ID, device ID, URL,
              referrer, and user agent).

          settings_by_rule_type: Configure logging settings for each rule type.

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not account_id:
            raise ValueError(f"Expected a non-empty value for `account_id` but received {account_id!r}")
        return await self._put(
            path_template("/accounts/{account_id}/gateway/logging", account_id=account_id),
            body=await async_maybe_transform(
                {
                    "redact_pii": redact_pii,
                    "settings_by_rule_type": settings_by_rule_type,
                },
                logging_update_params.LoggingUpdateParams,
            ),
            options=make_request_options(
                extra_headers=extra_headers,
                extra_query=extra_query,
                extra_body=extra_body,
                timeout=timeout,
                post_parser=ResultWrapper[Optional[LoggingSetting]]._unwrapper,
            ),
            cast_to=cast(Type[Optional[LoggingSetting]], ResultWrapper[LoggingSetting]),
        )

    async def get(
        self,
        *,
        account_id: str,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> Optional[LoggingSetting]:
        """
        Retrieve the current logging settings for the Zero Trust account.

        Args:
          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not account_id:
            raise ValueError(f"Expected a non-empty value for `account_id` but received {account_id!r}")
        return await self._get(
            path_template("/accounts/{account_id}/gateway/logging", account_id=account_id),
            options=make_request_options(
                extra_headers=extra_headers,
                extra_query=extra_query,
                extra_body=extra_body,
                timeout=timeout,
                post_parser=ResultWrapper[Optional[LoggingSetting]]._unwrapper,
            ),
            cast_to=cast(Type[Optional[LoggingSetting]], ResultWrapper[LoggingSetting]),
        )


class LoggingResourceWithRawResponse:
    def __init__(self, logging: LoggingResource) -> None:
        self._logging = logging

        self.update = to_raw_response_wrapper(
            logging.update,
        )
        self.get = to_raw_response_wrapper(
            logging.get,
        )


class AsyncLoggingResourceWithRawResponse:
    def __init__(self, logging: AsyncLoggingResource) -> None:
        self._logging = logging

        self.update = async_to_raw_response_wrapper(
            logging.update,
        )
        self.get = async_to_raw_response_wrapper(
            logging.get,
        )


class LoggingResourceWithStreamingResponse:
    def __init__(self, logging: LoggingResource) -> None:
        self._logging = logging

        self.update = to_streamed_response_wrapper(
            logging.update,
        )
        self.get = to_streamed_response_wrapper(
            logging.get,
        )


class AsyncLoggingResourceWithStreamingResponse:
    def __init__(self, logging: AsyncLoggingResource) -> None:
        self._logging = logging

        self.update = async_to_streamed_response_wrapper(
            logging.update,
        )
        self.get = async_to_streamed_response_wrapper(
            logging.get,
        )



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from __future__ import annotations

from typing import Union, Iterable
from typing_extensions import Literal, Required, Annotated, TypeAlias, TypedDict

from ...._types import SequenceNotStr
from ...._utils import PropertyInfo

__all__ = [
    "TelemetryQueryParams",
    "Timeframe",
    "Parameters",
    "ParametersCalculation",
    "ParametersFilter",
    "ParametersFilterUnionMember0",
    "ParametersFilterUnionMember0Filter",
    "ParametersFilterUnionMember0FilterUnionMember0",
    "ParametersFilterUnionMember0FilterWorkersObservabilityFilterLeaf",
    "ParametersFilterWorkersObservabilityFilterLeaf",
    "ParametersGroupBy",
    "ParametersHaving",
    "ParametersNeedle",
    "ParametersOrderBy",
]


class TelemetryQueryParams(TypedDict, total=False):
    account_id: Required[str]

    query_id: Required[Annotated[str, PropertyInfo(alias="queryId")]]
    """Identifier for the query.

    When parameters are omitted, this ID is used to load a previously saved query's
    parameters. When providing parameters inline, pass any identifier (e.g. an
    ad-hoc ID).
    """

    timeframe: Required[Timeframe]
    """Timeframe for the query using Unix timestamps in milliseconds.

    Narrower timeframes produce faster responses and more specific results.
    """

    chart: bool
    """When true, includes time-series data in the response."""

    compare: bool
    """
    When true, includes a comparison dataset from the previous time period of equal
    length.
    """

    dry: bool
    """When true, executes the query without persisting the results.

    Useful for validation or previewing.
    """

    granularity: float
    """Number of time-series buckets.

    Only used when view is 'calculations'. Omit to let the system auto-detect an
    appropriate granularity.
    """

    ignore_series: Annotated[bool, PropertyInfo(alias="ignoreSeries")]
    """
    When true, omits time-series data from the response and returns only aggregated
    values. Reduces response size when series are not needed.
    """

    limit: float
    """Maximum number of events to return when view is 'events'.

    Also controls the number of group-by rows when view is 'calculations'.
    """

    offset: str
    """Cursor for pagination in event, trace, and invocation views.

    Pass the $metadata.id of the last returned item to fetch the next page.
    """

    offset_by: Annotated[float, PropertyInfo(alias="offsetBy")]
    """Numeric offset for paginating grouped/pattern results (top-N lists).

    Use together with limit. Not used by cursor-based pagination.
    """

    offset_direction: Annotated[str, PropertyInfo(alias="offsetDirection")]
    """Pagination direction: 'next' for forward, 'prev' for backward."""

    parameters: Parameters
    """
    Query parameters defining what data to retrieve — filters, calculations,
    group-bys, and ordering. In practice this should always be provided for ad-hoc
    queries. Only omit when executing a previously saved query by queryId. Use the
    keys and values endpoints to discover available fields before building filters.
    """

    view: Literal["traces", "events", "calculations", "invocations", "requests", "agents"]
    """Controls the shape of the response.

    'events': individual log lines matching the query. 'calculations': aggregated
    metrics (count, avg, p99, etc.) with optional group-by breakdowns and
    time-series. 'invocations': events grouped by request ID. 'traces': distributed
    trace summaries. 'agents': Durable Object agent summaries.
    """


_TimeframeReservedKeywords = TypedDict(
    "_TimeframeReservedKeywords",
    {
        "from": float,
    },
    total=False,
)


class Timeframe(_TimeframeReservedKeywords, total=False):
    """Timeframe for the query using Unix timestamps in milliseconds.

    Narrower timeframes produce faster responses and more specific results.
    """

    to: Required[float]
    """End timestamp for the query timeframe (Unix timestamp in milliseconds)"""


class ParametersCalculation(TypedDict, total=False):
    operator: Required[
        Literal[
            "uniq",
            "count",
            "max",
            "min",
            "sum",
            "avg",
            "median",
            "p001",
            "p01",
            "p05",
            "p10",
            "p25",
            "p75",
            "p90",
            "p95",
            "p99",
            "p999",
            "stddev",
            "variance",
            "COUNT_DISTINCT",
            "COUNT",
            "MAX",
            "MIN",
            "SUM",
            "AVG",
            "MEDIAN",
            "P001",
            "P01",
            "P05",
            "P10",
            "P25",
            "P75",
            "P90",
            "P95",
            "P99",
            "P999",
            "STDDEV",
            "VARIANCE",
        ]
    ]
    """Aggregation operator to apply.

    Examples: count, avg, sum, min, max, p50, p90, p95, p99, uniq, stddev, variance.
    """

    alias: str
    """Custom label for this calculation in the results.

    Useful for distinguishing multiple calculations.
    """

    key: str
    """Field name to calculate over.

    Must exist in the data — verify with the keys endpoint. Omit for operators that
    don't require a key (e.g. count).
    """

    key_type: Annotated[Literal["string", "number", "boolean"], PropertyInfo(alias="keyType")]
    """Data type of the key.

    Required when key is provided to ensure correct aggregation.
    """


class ParametersFilterUnionMember0FilterUnionMember0(TypedDict, total=False):
    filter_combination: Required[Annotated[Literal["and", "or", "AND", "OR"], PropertyInfo(alias="filterCombination")]]

    filters: Required[Iterable[object]]

    kind: Required[Literal["group"]]


class ParametersFilterUnionMember0FilterWorkersObservabilityFilterLeaf(TypedDict, total=False):
    """A filter condition applied to query results.

    Use the keys and values endpoints to discover available fields and their values before constructing filters.
    """

    key: Required[str]
    """Filter field name.

    Use verified keys from previous query results or the keys endpoint. Common keys
    include $metadata.service, $metadata.origin, $metadata.trigger,
    $metadata.message, and $metadata.error.
    """

    operation: Required[
        Literal[
            "includes",
            "not_includes",
            "starts_with",
            "regex",
            "exists",
            "is_null",
            "in",
            "not_in",
            "eq",
            "neq",
            "gt",
            "gte",
            "lt",
            "lte",
            "=",
            "!=",
            ">",
            ">=",
            "<",
            "<=",
            "INCLUDES",
            "DOES_NOT_INCLUDE",
            "MATCH_REGEX",
            "EXISTS",
            "DOES_NOT_EXIST",
            "IN",
            "NOT_IN",
            "STARTS_WITH",
        ]
    ]
    """Comparison operator.

    String operators: includes, not_includes, starts_with, regex. Existence: exists,
    is_null. Set membership: in, not_in (comma-separated values). Numeric: eq, neq,
    gt, gte, lt, lte.
    """

    type: Required[Literal["string", "number", "boolean"]]
    """Data type of the filter field.

    Must match the actual type of the key being filtered.
    """

    kind: Literal["filter"]
    """Discriminator for leaf filter nodes.

    Always 'filter' when present; may be omitted.
    """

    value: Union[str, float, bool]
    """Comparison value.

    Must match actual values in your data — verify with the values endpoint. Ensure
    the value type (string/number/boolean) matches the field type. String
    comparisons are case-sensitive. Regex uses RE2 syntax (no
    lookaheads/lookbehinds).
    """


ParametersFilterUnionMember0Filter: TypeAlias = Union[
    ParametersFilterUnionMember0FilterUnionMember0, ParametersFilterUnionMember0FilterWorkersObservabilityFilterLeaf
]


class ParametersFilterUnionMember0(TypedDict, total=False):
    filter_combination: Required[Annotated[Literal["and", "or", "AND", "OR"], PropertyInfo(alias="filterCombination")]]

    filters: Required[Iterable[ParametersFilterUnionMember0Filter]]

    kind: Required[Literal["group"]]


class ParametersFilterWorkersObservabilityFilterLeaf(TypedDict, total=False):
    """A filter condition applied to query results.

    Use the keys and values endpoints to discover available fields and their values before constructing filters.
    """

    key: Required[str]
    """Filter field name.

    Use verified keys from previous query results or the keys endpoint. Common keys
    include $metadata.service, $metadata.origin, $metadata.trigger,
    $metadata.message, and $metadata.error.
    """

    operation: Required[
        Literal[
            "includes",
            "not_includes",
            "starts_with",
            "regex",
            "exists",
            "is_null",
            "in",
            "not_in",
            "eq",
            "neq",
            "gt",
            "gte",
            "lt",
            "lte",
            "=",
            "!=",
            ">",
            ">=",
            "<",
            "<=",
            "INCLUDES",
            "DOES_NOT_INCLUDE",
            "MATCH_REGEX",
            "EXISTS",
            "DOES_NOT_EXIST",
            "IN",
            "NOT_IN",
            "STARTS_WITH",
        ]
    ]
    """Comparison operator.

    String operators: includes, not_includes, starts_with, regex. Existence: exists,
    is_null. Set membership: in, not_in (comma-separated values). Numeric: eq, neq,
    gt, gte, lt, lte.
    """

    type: Required[Literal["string", "number", "boolean"]]
    """Data type of the filter field.

    Must match the actual type of the key being filtered.
    """

    kind: Literal["filter"]
    """Discriminator for leaf filter nodes.

    Always 'filter' when present; may be omitted.
    """

    value: Union[str, float, bool]
    """Comparison value.

    Must match actual values in your data — verify with the values endpoint. Ensure
    the value type (string/number/boolean) matches the field type. String
    comparisons are case-sensitive. Regex uses RE2 syntax (no
    lookaheads/lookbehinds).
    """


ParametersFilter: TypeAlias = Union[ParametersFilterUnionMember0, ParametersFilterWorkersObservabilityFilterLeaf]


class ParametersGroupBy(TypedDict, total=False):
    type: Required[Literal["string", "number", "boolean"]]
    """Data type of the group-by field."""

    value: Required[str]
    """Field name to group results by (e.g. $metadata.service, $metadata.statusCode)."""


class ParametersHaving(TypedDict, total=False):
    key: Required[str]
    """Calculation alias or operator to filter on after aggregation."""

    operation: Required[Literal["eq", "neq", "gt", "gte", "lt", "lte"]]
    """Numeric comparison operator: eq, neq, gt, gte, lt, lte."""

    value: Required[float]
    """Threshold value to compare the calculation result against."""


class ParametersNeedle(TypedDict, total=False):
    """Full-text search expression applied across all event fields.

    Matches events containing the specified text.
    """

    value: Required[Union[str, float, bool]]
    """The text or pattern to search for."""

    is_regex: Annotated[bool, PropertyInfo(alias="isRegex")]
    """When true, treats the value as a regular expression (RE2 syntax)."""

    match_case: Annotated[bool, PropertyInfo(alias="matchCase")]
    """When true, performs a case-sensitive search. Defaults to case-insensitive."""


class ParametersOrderBy(TypedDict, total=False):
    """Ordering for grouped calculation results.

    Only effective when a group-by is present.
    """

    value: Required[str]
    """Alias of the calculation to order results by.

    Must match the alias (or operator) of a calculation in the query.
    """

    order: Literal["asc", "desc"]
    """Sort direction: 'asc' for ascending, 'desc' for descending."""


class Parameters(TypedDict, total=False):
    """
    Query parameters defining what data to retrieve — filters, calculations, group-bys, and ordering. In practice this should always be provided for ad-hoc queries. Only omit when executing a previously saved query by queryId. Use the keys and values endpoints to discover available fields before building filters.
    """

    calculations: Iterable[ParametersCalculation]
    """Aggregation calculations to compute (e.g.

    count, avg, p99). Each calculation produces aggregate values and optional
    time-series data.
    """

    datasets: SequenceNotStr[str]
    """Datasets to query. Leave empty to query all available datasets."""

    filter_combination: Annotated[Literal["and", "or", "AND", "OR"], PropertyInfo(alias="filterCombination")]
    """
    Logical operator for combining top-level filters: 'and' (all must match) or 'or'
    (any must match). Defaults to 'and'.
    """

    filters: Iterable[ParametersFilter]
    """Filters to narrow query results.

    Use the keys and values endpoints to discover available fields before building
    filters. Supports nested groups via kind: 'group'. Maximum nesting depth is 4.
    """

    group_bys: Annotated[Iterable[ParametersGroupBy], PropertyInfo(alias="groupBys")]
    """Fields to group calculation results by.

    Only applicable when the query view is 'calculations'. Produces per-group
    aggregate values.
    """

    havings: Iterable[ParametersHaving]
    """Post-aggregation filters applied to calculation results.

    Use to filter groups after aggregation (e.g. only groups where count > 100).
    """

    limit: int
    """Maximum number of group-by rows to return in calculation results.

    A value of 10 is a sensible default for most use cases.
    """

    needle: ParametersNeedle
    """Full-text search expression applied across all event fields.

    Matches events containing the specified text.
    """

    order_by: Annotated[ParametersOrderBy, PropertyInfo(alias="orderBy")]
    """Ordering for grouped calculation results.

    Only effective when a group-by is present.
    """



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from typing import List, Optional
from typing_extensions import Literal

from pydantic import Field as FieldInfo

from ..._models import BaseModel

__all__ = ["NetworkPath", "Slot", "Sampling"]


class Slot(BaseModel):
    id: str
    """API Resource UUID tag."""

    client_to_app_rtt_ms: Optional[int] = FieldInfo(alias="clientToAppRttMs", default=None)
    """Round trip time in ms of the client to app mile"""

    client_to_cf_egress_rtt_ms: Optional[int] = FieldInfo(alias="clientToCfEgressRttMs", default=None)
    """Round trip time in ms of the client to Cloudflare egress mile"""

    client_to_cf_ingress_rtt_ms: Optional[int] = FieldInfo(alias="clientToCfIngressRttMs", default=None)
    """Round trip time in ms of the client to Cloudflare ingress mile"""

    timestamp: str

    client_to_isp_rtt_ms: Optional[int] = FieldInfo(alias="clientToIspRttMs", default=None)
    """Round trip time in ms of the client to ISP mile"""


class Sampling(BaseModel):
    """Specifies the sampling applied, if any, to the slots response.

    When sampled, results shown represent the first test run to the start of each sampling interval.
    """

    unit: Literal["hours"]

    value: int


class NetworkPath(BaseModel):
    slots: List[Slot]

    sampling: Optional[Sampling] = None
    """Specifies the sampling applied, if any, to the slots response.

    When sampled, results shown represent the first test run to the start of each
    sampling interval.
    """



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from typing import Optional

from ..._models import BaseModel

__all__ = ["SubscriptionZone"]


class SubscriptionZone(BaseModel):
    """A simple zone object. May have null properties if not a zone subscription."""

    id: Optional[str] = None
    """Identifier"""

    name: Optional[str] = None
    """The domain name"""



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from .layer3 import (
    Layer3Resource,
    AsyncLayer3Resource,
    Layer3ResourceWithRawResponse,
    AsyncLayer3ResourceWithRawResponse,
    Layer3ResourceWithStreamingResponse,
    AsyncLayer3ResourceWithStreamingResponse,
)
from .layer7 import (
    Layer7Resource,
    AsyncLayer7Resource,
    Layer7ResourceWithRawResponse,
    AsyncLayer7ResourceWithRawResponse,
    Layer7ResourceWithStreamingResponse,
    AsyncLayer7ResourceWithStreamingResponse,
)
from .attacks import (
    AttacksResource,
    AsyncAttacksResource,
    AttacksResourceWithRawResponse,
    AsyncAttacksResourceWithRawResponse,
    AttacksResourceWithStreamingResponse,
    AsyncAttacksResourceWithStreamingResponse,
)

__all__ = [
    "Layer3Resource",
    "AsyncLayer3Resource",
    "Layer3ResourceWithRawResponse",
    "AsyncLayer3ResourceWithRawResponse",
    "Layer3ResourceWithStreamingResponse",
    "AsyncLayer3ResourceWithStreamingResponse",
    "Layer7Resource",
    "AsyncLayer7Resource",
    "Layer7ResourceWithRawResponse",
    "AsyncLayer7ResourceWithRawResponse",
    "Layer7ResourceWithStreamingResponse",
    "AsyncLayer7ResourceWithStreamingResponse",
    "AttacksResource",
    "AsyncAttacksResource",
    "AttacksResourceWithRawResponse",
    "AsyncAttacksResourceWithRawResponse",
    "AttacksResourceWithStreamingResponse",
    "AsyncAttacksResourceWithStreamingResponse",
]



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from __future__ import annotations

from typing_extensions import Required, TypedDict

__all__ = ["PacfileCreateParams"]


class PacfileCreateParams(TypedDict, total=False):
    account_id: Required[str]

    contents: Required[str]
    """Actual contents of the PAC file"""

    name: Required[str]
    """Name of the PAC file."""

    description: str
    """Detailed description of the PAC file."""

    slug: str
    """URL-friendly version of the PAC file name.

    If not provided, it will be auto-generated
    """



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from .status import (
    StatusResource,
    AsyncStatusResource,
    StatusResourceWithRawResponse,
    AsyncStatusResourceWithRawResponse,
    StatusResourceWithStreamingResponse,
    AsyncStatusResourceWithStreamingResponse,
)
from .prefixes import (
    PrefixesResource,
    AsyncPrefixesResource,
    PrefixesResourceWithRawResponse,
    AsyncPrefixesResourceWithRawResponse,
    PrefixesResourceWithStreamingResponse,
    AsyncPrefixesResourceWithStreamingResponse,
)
from .allowlist import (
    AllowlistResource,
    AsyncAllowlistResource,
    AllowlistResourceWithRawResponse,
    AsyncAllowlistResourceWithRawResponse,
    AllowlistResourceWithStreamingResponse,
    AsyncAllowlistResourceWithStreamingResponse,
)
from .syn_protection import (
    SynProtectionResource,
    AsyncSynProtectionResource,
    SynProtectionResourceWithRawResponse,
    AsyncSynProtectionResourceWithRawResponse,
    SynProtectionResourceWithStreamingResponse,
    AsyncSynProtectionResourceWithStreamingResponse,
)
from .tcp_flow_protection import (
    TCPFlowProtectionResource,
    AsyncTCPFlowProtectionResource,
    TCPFlowProtectionResourceWithRawResponse,
    AsyncTCPFlowProtectionResourceWithRawResponse,
    TCPFlowProtectionResourceWithStreamingResponse,
    AsyncTCPFlowProtectionResourceWithStreamingResponse,
)
from .advanced_tcp_protection import (
    AdvancedTCPProtectionResource,
    AsyncAdvancedTCPProtectionResource,
    AdvancedTCPProtectionResourceWithRawResponse,
    AsyncAdvancedTCPProtectionResourceWithRawResponse,
    AdvancedTCPProtectionResourceWithStreamingResponse,
    AsyncAdvancedTCPProtectionResourceWithStreamingResponse,
)

__all__ = [
    "AllowlistResource",
    "AsyncAllowlistResource",
    "AllowlistResourceWithRawResponse",
    "AsyncAllowlistResourceWithRawResponse",
    "AllowlistResourceWithStreamingResponse",
    "AsyncAllowlistResourceWithStreamingResponse",
    "PrefixesResource",
    "AsyncPrefixesResource",
    "PrefixesResourceWithRawResponse",
    "AsyncPrefixesResourceWithRawResponse",
    "PrefixesResourceWithStreamingResponse",
    "AsyncPrefixesResourceWithStreamingResponse",
    "SynProtectionResource",
    "AsyncSynProtectionResource",
    "SynProtectionResourceWithRawResponse",
    "AsyncSynProtectionResourceWithRawResponse",
    "SynProtectionResourceWithStreamingResponse",
    "AsyncSynProtectionResourceWithStreamingResponse",
    "TCPFlowProtectionResource",
    "AsyncTCPFlowProtectionResource",
    "TCPFlowProtectionResourceWithRawResponse",
    "AsyncTCPFlowProtectionResourceWithRawResponse",
    "TCPFlowProtectionResourceWithStreamingResponse",
    "AsyncTCPFlowProtectionResourceWithStreamingResponse",
    "StatusResource",
    "AsyncStatusResource",
    "StatusResourceWithRawResponse",
    "AsyncStatusResourceWithRawResponse",
    "StatusResourceWithStreamingResponse",
    "AsyncStatusResourceWithStreamingResponse",
    "AdvancedTCPProtectionResource",
    "AsyncAdvancedTCPProtectionResource",
    "AdvancedTCPProtectionResourceWithRawResponse",
    "AsyncAdvancedTCPProtectionResourceWithRawResponse",
    "AdvancedTCPProtectionResourceWithStreamingResponse",
    "AsyncAdvancedTCPProtectionResourceWithStreamingResponse",
]



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from __future__ import annotations

from typing_extensions import TypedDict

__all__ = ["NotificationSettingsParam"]


class NotificationSettingsParam(TypedDict, total=False):
    """Configure the message the user's device shows during an antivirus scan."""

    enabled: bool
    """Specify whether to enable notifications."""

    include_context: bool
    """Specify whether to include context information as query parameters."""

    msg: str
    """Specify the message to show in the notification."""

    support_url: str
    """Specify a URL that directs users to more information.

    If unset, the notification opens a block page.
    """



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from typing import Optional
from datetime import datetime
from typing_extensions import Literal

from ...._models import BaseModel

__all__ = ["AllowPolicyCreateResponse"]


class AllowPolicyCreateResponse(BaseModel):
    """An email allow policy"""

    id: str
    """Allow policy identifier"""

    created_at: datetime

    last_modified: datetime
    """Deprecated, use `modified_at` instead. End of life: November 1, 2026."""

    comments: Optional[str] = None

    is_acceptable_sender: Optional[bool] = None
    """
    Messages from this sender will be exempted from Spam, Spoof and Bulk
    dispositions. Note - This will not exempt messages with Malicious or Suspicious
    dispositions.
    """

    is_exempt_recipient: Optional[bool] = None
    """Messages to this recipient will bypass all detections"""

    is_recipient: Optional[bool] = None
    """Deprecated as of July 1, 2025.

    Use `is_exempt_recipient` instead. End of life: July 1, 2026.
    """

    is_regex: Optional[bool] = None

    is_sender: Optional[bool] = None
    """Deprecated as of July 1, 2025.

    Use `is_trusted_sender` instead. End of life: July 1, 2026.
    """

    is_spoof: Optional[bool] = None
    """Deprecated as of July 1, 2025.

    Use `is_acceptable_sender` instead. End of life: July 1, 2026.
    """

    is_trusted_sender: Optional[bool] = None
    """Messages from this sender will bypass all detections and link following"""

    modified_at: Optional[datetime] = None

    pattern: Optional[str] = None

    pattern_type: Optional[Literal["EMAIL", "DOMAIN", "IP", "UNKNOWN"]] = None
    """
    Type of pattern matching. Note: UNKNOWN is deprecated and cannot be used when
    creating or updating policies, but may be returned for existing entries.
    """

    verify_sender: Optional[bool] = None
    """Enforce DMARC, SPF or DKIM authentication.

    When on, Email Security only honors policies that pass authentication.
    """



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from typing import Optional

from ...._models import BaseModel

__all__ = ["WANStaticAddressing"]


class WANStaticAddressing(BaseModel):
    """(optional) if omitted, use DHCP.

    Submit secondary_address when site is in high availability mode.
    """

    address: str
    """A valid CIDR notation representing an IP range."""

    gateway_address: str
    """A valid IPv4 address."""

    secondary_address: Optional[str] = None
    """A valid CIDR notation representing an IP range."""



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from __future__ import annotations

from .credential_update_params import CredentialUpdateParams as CredentialUpdateParams
from .credential_update_response import CredentialUpdateResponse as CredentialUpdateResponse



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from typing import Optional
from datetime import datetime

from .scope import Scope
from ..._models import BaseModel

__all__ = ["RouteGetResponse", "Route"]


class Route(BaseModel):
    id: str
    """Identifier"""

    nexthop: str
    """The next-hop IP Address for the static route."""

    prefix: str
    """IP Prefix in Classless Inter-Domain Routing format."""

    priority: int
    """Priority of the static route."""

    created_on: Optional[datetime] = None
    """When the route was created."""

    description: Optional[str] = None
    """An optional human provided description of the static route."""

    modified_on: Optional[datetime] = None
    """When the route was last modified."""

    scope: Optional[Scope] = None
    """Used only for ECMP routes."""

    weight: Optional[int] = None
    """Optional weight of the ECMP scope - if provided."""


class RouteGetResponse(BaseModel):
    route: Optional[Route] = None



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from __future__ import annotations

from .rules import (
    RulesResource,
    AsyncRulesResource,
    RulesResourceWithRawResponse,
    AsyncRulesResourceWithRawResponse,
    RulesResourceWithStreamingResponse,
    AsyncRulesResourceWithStreamingResponse,
)
from ..._compat import cached_property
from .site_info import (
    SiteInfoResource,
    AsyncSiteInfoResource,
    SiteInfoResourceWithRawResponse,
    AsyncSiteInfoResourceWithRawResponse,
    SiteInfoResourceWithStreamingResponse,
    AsyncSiteInfoResourceWithStreamingResponse,
)
from ..._resource import SyncAPIResource, AsyncAPIResource

__all__ = ["RUMResource", "AsyncRUMResource"]


class RUMResource(SyncAPIResource):
    @cached_property
    def site_info(self) -> SiteInfoResource:
        return SiteInfoResource(self._client)

    @cached_property
    def rules(self) -> RulesResource:
        return RulesResource(self._client)

    @cached_property
    def with_raw_response(self) -> RUMResourceWithRawResponse:
        """
        This property can be used as a prefix for any HTTP method call to return
        the raw response object instead of the parsed content.

        For more information, see https://www.github.com/cloudflare/cloudflare-python#accessing-raw-response-data-eg-headers
        """
        return RUMResourceWithRawResponse(self)

    @cached_property
    def with_streaming_response(self) -> RUMResourceWithStreamingResponse:
        """
        An alternative to `.with_raw_response` that doesn't eagerly read the response body.

        For more information, see https://www.github.com/cloudflare/cloudflare-python#with_streaming_response
        """
        return RUMResourceWithStreamingResponse(self)


class AsyncRUMResource(AsyncAPIResource):
    @cached_property
    def site_info(self) -> AsyncSiteInfoResource:
        return AsyncSiteInfoResource(self._client)

    @cached_property
    def rules(self) -> AsyncRulesResource:
        return AsyncRulesResource(self._client)

    @cached_property
    def with_raw_response(self) -> AsyncRUMResourceWithRawResponse:
        """
        This property can be used as a prefix for any HTTP method call to return
        the raw response object instead of the parsed content.

        For more information, see https://www.github.com/cloudflare/cloudflare-python#accessing-raw-response-data-eg-headers
        """
        return AsyncRUMResourceWithRawResponse(self)

    @cached_property
    def with_streaming_response(self) -> AsyncRUMResourceWithStreamingResponse:
        """
        An alternative to `.with_raw_response` that doesn't eagerly read the response body.

        For more information, see https://www.github.com/cloudflare/cloudflare-python#with_streaming_response
        """
        return AsyncRUMResourceWithStreamingResponse(self)


class RUMResourceWithRawResponse:
    def __init__(self, rum: RUMResource) -> None:
        self._rum = rum

    @cached_property
    def site_info(self) -> SiteInfoResourceWithRawResponse:
        return SiteInfoResourceWithRawResponse(self._rum.site_info)

    @cached_property
    def rules(self) -> RulesResourceWithRawResponse:
        return RulesResourceWithRawResponse(self._rum.rules)


class AsyncRUMResourceWithRawResponse:
    def __init__(self, rum: AsyncRUMResource) -> None:
        self._rum = rum

    @cached_property
    def site_info(self) -> AsyncSiteInfoResourceWithRawResponse:
        return AsyncSiteInfoResourceWithRawResponse(self._rum.site_info)

    @cached_property
    def rules(self) -> AsyncRulesResourceWithRawResponse:
        return AsyncRulesResourceWithRawResponse(self._rum.rules)


class RUMResourceWithStreamingResponse:
    def __init__(self, rum: RUMResource) -> None:
        self._rum = rum

    @cached_property
    def site_info(self) -> SiteInfoResourceWithStreamingResponse:
        return SiteInfoResourceWithStreamingResponse(self._rum.site_info)

    @cached_property
    def rules(self) -> RulesResourceWithStreamingResponse:
        return RulesResourceWithStreamingResponse(self._rum.rules)


class AsyncRUMResourceWithStreamingResponse:
    def __init__(self, rum: AsyncRUMResource) -> None:
        self._rum = rum

    @cached_property
    def site_info(self) -> AsyncSiteInfoResourceWithStreamingResponse:
        return AsyncSiteInfoResourceWithStreamingResponse(self._rum.site_info)

    @cached_property
    def rules(self) -> AsyncRulesResourceWithStreamingResponse:
        return AsyncRulesResourceWithStreamingResponse(self._rum.rules)
